// -*- C++ -*-
/*!
 * @file ActivatorComp.cpp
 * @brief Activator application
 * @date $Date: 2008-01-13 07:24:08 $
 *
 * Copyright (c) 2003-2007 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id: ActivatorComp.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include <vector>
#include <string>
#include <rtm/CorbaNaming.h>
#include <rtm/RTObject.h>
#include <rtm/NVUtil.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/RTMUtil.h>
#include <assert.h>

#include <ActivatorTask.h>

extern "C" {
#include <OpenRTMSetup.h>
}

#define	RTM_DEFAULT_NAME_SERVER		"10.0.0.11:30001"
#define	ACTIVATOR_PORT			"30000"

using namespace RTC;

void rtm_act_main_task(intptr_t exinf)
{
  char server_addr[64] = "";
  int _argc = 3;
  char *_argv[10];
  _argv[0] = "ActivatorComp";
  _argv[1] = "-ORBServerPort";
  _argv[2] = ACTIVATOR_PORT;

  CorbaConsumer<RTObject> conin;

  getNameServerAddr( (int8_t*)server_addr );
  if ( server_addr[0] == '\0' ) {
    strcpy( server_addr, RTM_DEFAULT_NAME_SERVER );
  }

  CORBA::ORB_var orb = CORBA::ORB_init(_argc, _argv);
  CorbaNaming naming(orb, server_addr);

  // find HelloRTWorld0 component
  conin.setObject(naming.resolve("HelloRTWorld0.rtc"));

  // activate HelloRTWorld0

  ExecutionContextServiceList_var eclisti;
  eclisti = conin->get_execution_context_services();

  eclisti[(CORBA::ULong)0]->activate_component(RTObject::_duplicate(conin._ptr()));

  orb->destroy();

  RTM_MESSAGE_OUT1( "Activated Component\n" );

  ext_tsk();
}
