// -*- C++ -*-
/*!
 * @file  HelloRTWorldComp.cpp
 * @brief Standalone component
 * @date  $Date: 2008-01-14 07:43:44 $
 *
 * $Id: HelloRTWorldComp.cpp 778 2008-07-29 04:59:25Z n-ando $
 */

#include <rtm/Manager.h>
#include <string>
#include "HelloRTWorld.h"
#include "ethmac.h"

void HRWModuleInit(RTC::Manager* manager)
{
  HelloRTWorldInit(manager);
  RTC::RtcBase* comp;

  HRWExCnt = new RTC::PeriodicExecutionContext( RTM_HRW_EXCNT_TASK );

  // Create a component
  comp = manager->createComponent("HelloRTWorld", (RTC::ExecutionContextBase *)HRWExCnt );
  RTM_MESSAGE_OUT1("succeed.\n");

  HRWExCnt->set_rate(1);

  RTC::ComponentProfile_var prof;
  prof = comp->get_component_profile();
  RTM_MESSAGE_OUT1("=================================================\n");
  RTM_MESSAGE_OUT1(" Component Profile\n");
  RTM_MESSAGE_OUT1("-------------------------------------------------\n");
  RTM_MESSAGE_OUT2("InstanceID:     %s\n",prof->instance_name);
  RTM_MESSAGE_OUT2("Implementation: %s\n",prof->type_name);
  RTM_MESSAGE_OUT2("Description:    %s\n",prof->description);
  RTM_MESSAGE_OUT2("Version:        %s\n",prof->version);
  RTM_MESSAGE_OUT2("Maker:          %s\n",prof->vendor);
  RTM_MESSAGE_OUT2("Category:       %s\n",prof->category);
  RTM_MESSAGE_OUT1("  Other properties   \n");
  NVUtil::dump(prof->properties);
  RTM_MESSAGE_OUT1("=================================================\n");

  return;
}

void rtm_hrw_main_task(intptr_t exinf)
{
  int argc = 1;
  char *argv[10];
  argv[0] = "HelloRTWorld";

  RTC::Manager* manager;
  manager = RTC::Manager::init(argc, argv, &TimerTask, &RunnerTask, get_endp_portno() );

  // Set module initialization proceduer
  // This procedure will be invoked in activateManager() function.
  manager->setModuleInitProc(HRWModuleInit);

  // Activate manager and register to naming service
  manager->activateManager();

  sig_sem( RTM_WAIT_HRW_SEM );

  // run the manager in blocking mode
  // runManager(false) is the default.
//manager->runManager();

  // If you want to run the manager in non-blocking mode, do like this
  manager->runManager(true);

  ext_tsk();
}
