// -*- C++ -*-
/*!
 * @file  LEDOut.cpp
 * @brief LED output component
 * $Date: 2007-10-09 07:33:08 $
 *
 * $Id: LEDOut.cpp 775 2008-07-28 16:14:45Z n-ando $
 */

#include <LEDOut.h>
#include <coil/Time.h>
extern "C" {
#include "led.h"
}

// Module specification
// <rtc-template block="module_spec">
static const char* ledout_spec[] =
  {
    "implementation_id", "LEDOut",
    "type_name",         "LEDOut",
    "description",       "LED output component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

LEDOut::LEDOut(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_inIn("in", m_in),
    // </rtc-template>

	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

LEDOut::~LEDOut()
{
}


RTC::ReturnCode_t LEDOut::onInitialize()
{
  registerInPort("in", m_inIn);

  init_led();
  set_led( LED_R, false );
  set_led( LED_Y, false );
  return RTC::RTC_OK;
}

RTC::ReturnCode_t LEDOut::onExecute(RTC::UniqueId ec_id)
{
  if (m_inIn.isNew())
    {
      m_inIn.read();

      if ( m_in.data >= 0 && m_in.data <= 3 )
        {
          set_led( LED_R, ( m_in.data & 1 ) );
          set_led( LED_Y, ( m_in.data & 2 ) );
        }
    }

  return RTC::RTC_OK;
}


extern "C"
{
 
  void LEDOutInit(RTC::Manager* manager)
  {
    RTC::Properties profile(ledout_spec);
    manager->registerFactory(profile,
                             RTC::Create<LEDOut>,
                             RTC::Delete<LEDOut>);
  }
  
};
