// -*- C++ -*-
/*!
 * @file  SWIn.cpp
 * @brief SW input component
 * $Date: 2007-10-09 07:33:04 $
 *
 * $Id: SWIn.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include "SWIn.h"

extern "C" {
#include "switch.h"
}

#define RTM_SWIN_PORTID	1

extern bool g_sw1;
extern bool g_sw2;

// Module specification
// <rtc-template block="module_spec">
static const char* swin_spec[] =
  {
    "implementation_id", "SWIn",
    "type_name",         "SWIn",
    "description",       "SW input component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

SWIn::SWIn(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_outOut("out", m_out),
    // </rtc-template>
    m_sw1(false),
    m_sw2(false),
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

SWIn::~SWIn()
{
}


RTC::ReturnCode_t SWIn::onInitialize()
{
  registerOutPort("out", m_outOut);

  init_switch();

  return RTC::RTC_OK;
}
RTC::ReturnCode_t SWIn::onExecute(RTC::UniqueId ec_id)
{
  bool current_sw1;
  bool current_sw2;
  long sw_status = 0;

  watch_switch();

  RTM_CRITICAL_SECTION_START
  current_sw1 = g_sw1;
  current_sw2 = g_sw2;
  RTM_CRITICAL_SECTION_END

  if ( current_sw1 != m_sw1 || current_sw2 != m_sw2 )
  {
    if ( current_sw1 )
    {
      sw_status += 1;
    }
    if ( current_sw2 )
    {
      sw_status += 2;
    }

    m_sw1 = current_sw1;
    m_sw2 = current_sw2;

    m_out.data = sw_status;
    m_outOut.write();
  }

  return RTC::RTC_OK;
}


extern "C"
{
 
  void SWInInit(RTC::Manager* manager)
  {
    RTC::Properties profile(swin_spec);
    manager->registerFactory(profile,
                             RTC::Create<SWIn>,
                             RTC::Delete<SWIn>);
  }
  
};
