// -*- C++ -*-
/*!
 * @file ConnectorComp.cpp
 * @brief connector application
 * @date $Date: 2008-01-13 07:24:08 $
 *
 * Copyright (c) 2003-2007 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id: ConnectorComp.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include <vector>
#include <string>
#include <rtm/CorbaNaming.h>
#include <rtm/RTObject.h>
#include <rtm/NVUtil.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/RTMUtil.h>
#include <assert.h>

#include <ConnectorTask.h>

extern "C" {
#include <OpenRTMSetup.h>
}

#define	RTM_DEFAULT_NAME_SERVER		"10.0.0.11:30001"
#define	CONNECTOR_PORT			"30000"

using namespace RTC;

void rtm_con_main_task(intptr_t exinf)
{
  char server_addr[64] = "";
  int _argc = 3;
  char *_argv[10];
  _argv[0] = "ConnectorComp";
  _argv[1] = "-ORBServerPort";
  _argv[2] = CONNECTOR_PORT;

  std::string subs_type;
  std::string period;

//subs_type = "Flush";
  subs_type = "New";
  period = "1.0";

  RTM_MESSAGE_OUT2( "Subscription Type: %s\n", subs_type.c_str() );
  if (period != "")
    RTM_MESSAGE_OUT2( "Period: %s [Hz]\n", period.c_str() );

  getNameServerAddr( (int8_t*)server_addr );
  if ( server_addr[0] == '\0' ) {
    strcpy( server_addr, RTM_DEFAULT_NAME_SERVER );
  }

  CORBA::ORB_var orb = CORBA::ORB_init(_argc, _argv);
  CorbaNaming naming(orb, server_addr);

  CorbaConsumer<RTObject> conin, conout;
  PortList_var pin;
  PortList_var pout;

  // find ConsoleIn0 component
  conin.setObject(naming.resolve("ConsoleIn0.rtc"));

  // get ports
  pin = conin->get_ports();

  pin[(CORBA::ULong)0]->disconnect_all();
  assert(pin->length() > 0);
  // activate ConsoleIn0

  ExecutionContextServiceList_var eclisti;
  eclisti = conin->get_execution_context_services();

  eclisti[(CORBA::ULong)0]->activate_component(RTObject::_duplicate(conin._ptr()));

  // find ConsoleOut0 component
  conout.setObject(naming.resolve("ConsoleOut0.rtc"));

  // get ports
  pout = conout->get_ports();
  pout[(CORBA::ULong)0]->disconnect_all();
  assert(pout->length() > 0);
  // activate ConsoleOut0
  ExecutionContextServiceList_var eclisto;
  eclisto = conout->get_execution_context_services();
  eclisto[(CORBA::ULong)0]->activate_component(RTObject::_duplicate(conout._ptr()));

  // connect ports
  ConnectorProfile prof;
  prof.connector_id = "";
  prof.name = CORBA::string_dup("connector0");
  prof.ports.length(2);
  prof.ports[0] = pin[(CORBA::ULong)0];
  prof.ports[1] = pout[(CORBA::ULong)0];

  CORBA_SeqUtil::push_back(prof.properties,
			   NVUtil::newNV("dataport.interface_type",
					 "CORBA_Any"));
  CORBA_SeqUtil::push_back(prof.properties,
			   NVUtil::newNV("dataport.dataflow_type",
					 "Push"));
  CORBA_SeqUtil::push_back(prof.properties,
			   NVUtil::newNV("dataport.subscription_type",
					 subs_type.c_str()));
  if (period != "")
  CORBA_SeqUtil::push_back(prof.properties,
			   NVUtil::newNV("dataport.push_interval",
					 period.c_str()));

  ReturnCode_t ret;
  ret = pin[(CORBA::ULong)0]->connect(prof);
  assert(ret == RTC::RTC_OK);

  RTM_MESSAGE_OUT2( "Connector ID: %s\n", prof.connector_id );
  NVUtil::dump(prof.properties);

  orb->destroy();

  ext_tsk();
}
