// -*- C++ -*-
/*!
 * @file  ConsoleIn.cpp
 * @brief Console input component
 * $Date: 2007-10-09 07:33:04 $
 *
 * $Id: ConsoleIn.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include "ConsoleIn.h"

#define RTM_CIN_PORTID	1


// Module specification
// <rtc-template block="module_spec">
static const char* consolein_spec[] =
  {
    "implementation_id", "ConsoleIn",
    "type_name",         "ConsoleIn",
    "description",       "Console input component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ConsoleIn::ConsoleIn(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_outOut("out", m_out),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ConsoleIn::~ConsoleIn()
{
}


RTC::ReturnCode_t ConsoleIn::onInitialize()
{
  registerOutPort("out", m_outOut);
  m_out.tm.sec  = 0;
  m_out.tm.nsec = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t ConsoleIn::onExecute(RTC::UniqueId ec_id)
{
  long in_data = 0;
  long digit = 0;
  int is_err = 0;
  char_t c;

  RTM_MESSAGE_OUT1( "Please input number:" );

  while (1)
  {
    serial_rea_dat( RTM_CIN_PORTID, &c, 1);
    if ( c == '\r' || c == '\n' )
    {
      break;
    }

    if ( c < '0' || c > '9' )
    {
      is_err = 1;
      break;
    }

    in_data *= 10;
    in_data += ( c - '0' );
    digit++;
  }

  RTM_MESSAGE_OUT1( "\n" );

  if ( is_err == 0 && digit > 0 )
  {
    m_out.data = in_data;
    RTM_MESSAGE_OUT2( "Sending to subscriber:%d\n", m_out.data );
    m_outOut.write();
  }

  return RTC::RTC_OK;
}


extern "C"
{
 
  void ConsoleInInit(RTC::Manager* manager)
  {
    RTC::Properties profile(consolein_spec);
    manager->registerFactory(profile,
                             RTC::Create<ConsoleIn>,
                             RTC::Delete<ConsoleIn>);
  }
  
};


