// -*- C++ -*-
/*!
 * @file ConsoleInComp.cpp
 * @brief Standalone component
 * @date $Date: 2008-01-14 07:43:41 $
 *
 * $Id: ConsoleInComp.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include <rtm/Manager.h>
#include <string>
#include "ConsoleIn.h"
#include <rtm/NVUtil.h>
#include "ethmac.h"

void CInModuleInit(RTC::Manager* manager)
{
  ConsoleInInit(manager);
  RTC::RtcBase* comp;

  CInExCnt = new RTC::PeriodicExecutionContext( RTM_CIN_EXCNT_TASK );

  // Create a component
  RTM_MESSAGE_OUT1( "Creating a component: \"ConsoleIn\"....\n" );
  comp = manager->createComponent("ConsoleIn", (RTC::ExecutionContextBase *)CInExCnt );
  RTM_MESSAGE_OUT1( "succeed.\n" );

  RTC::ComponentProfile_var prof;
  prof = comp->get_component_profile();
  RTM_MESSAGE_OUT1("=================================================\n");
  RTM_MESSAGE_OUT1(" Component Profile\n");
  RTM_MESSAGE_OUT1("-------------------------------------------------\n");
  RTM_MESSAGE_OUT2("InstanceID:     %s\n", prof->instance_name );
  RTM_MESSAGE_OUT2("Implementation: %s\n", prof->type_name );
  RTM_MESSAGE_OUT2("Description:    %s\n", prof->description );
  RTM_MESSAGE_OUT2("Version:        %s\n", prof->version );
  RTM_MESSAGE_OUT2("Maker:          %s\n", prof->vendor );
  RTM_MESSAGE_OUT2("Category:       %s\n", prof->category );
  RTM_MESSAGE_OUT1("  Other properties   \n");
  NVUtil::dump(prof->properties);
  RTM_MESSAGE_OUT1("=================================================\n");

  PortList* portlist;
  portlist = comp->get_ports();

  for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
    {
      Port_ptr port;
      port = (*portlist)[i];
      RTM_MESSAGE_OUT1( "=================================================\n");
      RTM_MESSAGE_OUT3( "Port%d (name): %s\n", i, port->get_port_profile()->name );
      RTM_MESSAGE_OUT1( "-------------------------------------------------\n" );

      RTC::PortInterfaceProfileList iflist;
      iflist = port->get_port_profile()->interfaces;

      for (CORBA::ULong i(0), n(iflist.length()); i < n; ++i)
	{
          RTM_MESSAGE_OUT2( "I/F name: %s\n", iflist[i].instance_name );
          RTM_MESSAGE_OUT2( "I/F type: %s\n", iflist[i].type_name );

	  const char* pol;
	  pol = iflist[i].polarity == 0 ? "PROVIDED" : "REQUIRED";
	  RTM_MESSAGE_OUT2( "Polarity: %s\n", pol );
	}
      RTM_MESSAGE_OUT1( "- properties -\n" );
      NVUtil::dump(port->get_port_profile()->properties);
      RTM_MESSAGE_OUT1( "-------------------------------------------------\n" );
    }

  return;
}


void rtm_cin_main_task(intptr_t exinf)
{
  int argc = 1;
  char *argv[10];
  argv[0] = "ConsoleIn";

  RTC::Manager* manager;
  manager = RTC::Manager::init(argc, argv, &TimerTask, &RunnerTask, get_endp_portno() );

  // Set module initialization proceduer
  // This procedure will be invoked in activateManager() function.
  manager->setModuleInitProc(CInModuleInit);

  // Activate manager and register to naming service
  manager->activateManager();

  sig_sem( RTM_WAIT_CIN_SEM );

  // run the manager in blocking mode
  // runManager(false) is the default.
//manager->runManager();

  // If you want to run the manager in non-blocking mode, do like this
  manager->runManager(true);

  ext_tsk();
}
