// -*- C++ -*-
/*!
 * @file  StringIn.cpp
 * @brief String input component
 * $Date: 2007-10-09 07:33:04 $
 *
 * $Id: StringIn.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include "StringIn.h"

#define RTM_SIN_PORTID	1


// Module specification
// <rtc-template block="module_spec">
static const char* stringin_spec[] =
  {
    "implementation_id", "StringIn",
    "type_name",         "StringIn",
    "description",       "String input component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

StringIn::StringIn(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_string_outOut("string_out", m_string_out),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

StringIn::~StringIn()
{
}

RTC::ReturnCode_t StringIn::onInitialize()
{
  registerOutPort("string_out", m_string_outOut);
  return RTC::RTC_OK;
}

RTC::ReturnCode_t StringIn::onExecute(RTC::UniqueId ec_id)
{
  long in_data = 0;
  char_t c;
  std::string s;

  RTM_MESSAGE_OUT1( "Please input string: " );

  while (1)
  {
    serial_rea_dat( RTM_SIN_PORTID, &c, 1);
    if ( c == '\r' || c == '\n' )
    {
      break;
    }

    s += c;
  }

  RTM_MESSAGE_OUT1( "\n" );

  if ( s.size() > 0 )
  {
    m_string_out.data = CORBA::string_dup( s.c_str() );
    RTM_MESSAGE_OUT2( "Sending to subscriber:%s\n", m_string_out.data );
    m_string_outOut.write();
  }

  return RTC::RTC_OK;
}


extern "C"
{
 
  void StringInInit(RTC::Manager* manager)
  {
    RTC::Properties profile(stringin_spec);
    manager->registerFactory(profile,
                             RTC::Create<StringIn>,
                             RTC::Delete<StringIn>);
  }
  
};


