// -*- C++ -*-
/*!
 * @file  StringOutComp.cpp
 * @brief Standalone component
 * @date  $Date: 2008-01-14 07:43:44 $
 *
 * $Id: StringOutComp.cpp 778 2008-07-29 04:59:25Z n-ando $
 */

#include <rtm/Manager.h>
#include <string>
#include "StringOut.h"
#include "ethmac.h"

void SOutModuleInit(RTC::Manager* manager)
{
  StringOutInit(manager);
  RTC::RtcBase* comp;

  SOutExCnt = new RTC::PeriodicExecutionContext( RTM_SOUT_EXCNT_TASK );

  // Create a component
  comp = manager->createComponent("StringOut", (RTC::ExecutionContextBase *)SOutExCnt );
  RTM_MESSAGE_OUT1("succeed.\n");

  RTC::ComponentProfile_var prof;
  prof = comp->get_component_profile();
  RTM_MESSAGE_OUT1("=================================================\n");
  RTM_MESSAGE_OUT1(" Component Profile\n");
  RTM_MESSAGE_OUT1("-------------------------------------------------\n");
  RTM_MESSAGE_OUT2("InstanceID:     %s\n",prof->instance_name);
  RTM_MESSAGE_OUT2("Implementation: %s\n",prof->type_name);
  RTM_MESSAGE_OUT2("Description:    %s\n",prof->description);
  RTM_MESSAGE_OUT2("Version:        %s\n",prof->version);
  RTM_MESSAGE_OUT2("Maker:          %s\n",prof->vendor);
  RTM_MESSAGE_OUT2("Category:       %s\n",prof->category);
  RTM_MESSAGE_OUT1("  Other properties   \n");
  NVUtil::dump(prof->properties);
  RTM_MESSAGE_OUT1("=================================================\n");

  PortList* portlist;
  portlist = comp->get_ports();

  for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
    {
      Port_ptr port;
      port = (*portlist)[i];
      RTM_MESSAGE_OUT1( "=================================================\n" );
      RTM_MESSAGE_OUT3( "Port%d (name): %s\n", i, port->get_port_profile()->name );
      RTM_MESSAGE_OUT1( "-------------------------------------------------\n" );
    
      RTC::PortInterfaceProfileList iflist;
      iflist = port->get_port_profile()->interfaces;

      for (CORBA::ULong i(0), n(iflist.length()); i < n; ++i)
	{
          RTM_MESSAGE_OUT2( "I/F name: %s\n", iflist[i].instance_name );
          RTM_MESSAGE_OUT2( "I/F type: %s\n", iflist[i].type_name );

	  const char* pol;
	  pol = iflist[i].polarity == 0 ? "PROVIDED" : "REQUIRED";
          RTM_MESSAGE_OUT2( "Polarity: %s\n", pol );
	}

      RTM_MESSAGE_OUT1( "- properties -\n" );
      NVUtil::dump(port->get_port_profile()->properties);
      RTM_MESSAGE_OUT1( "-------------------------------------------------\n" );
    }

  return;
}

void rtm_sout_main_task(intptr_t exinf)
{
  int argc = 1;
  char *argv[10];
  argv[0] = "StringOut";

  RTC::Manager* manager;
  manager = RTC::Manager::init(argc, argv, &TimerTask, &RunnerTask, get_endp_portno() );

  // Set module initialization proceduer
  // This procedure will be invoked in activateManager() function.
  manager->setModuleInitProc(SOutModuleInit);

  // Activate manager and register to naming service
  manager->activateManager();

  sig_sem( RTM_WAIT_SOUT_SEM );

  // run the manager in blocking mode
  // runManager(false) is the default.
//manager->runManager();

  // If you want to run the manager in non-blocking mode, do like this
  manager->runManager(true);

  ext_tsk();
}
