// -*- C++ -*-
/*!
 * @file  PublisherFactory.cpp
 * @brief PublisherFactory class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherFactory.cpp 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#include <rtm/PublisherFactory.h>
#include <rtm/PublisherNew.h>
#include <rtm/PublisherPeriodic.h>
#include <rtm/PublisherFlush.h>
#include <rtm/Properties.h>
#include <rtm/NVUtil.h>
#include <rtm/InPortConsumer.h>
#include <string>
#include <PublisherNewInfo.h>
#include <PublisherPeriodicInfo.h>
#include <rtm/RTMUtil.h>

namespace RTC
{
  /*!
   * @if jp
   * @brief Publisher
   * @else
   * @brief Create the Publishers
   * @endif
   */
  PublisherBase* PublisherFactory::create(InPortConsumer* consumer,
					  Properties& property)
  {
    int index;
    std::string pub_type;
    pub_type = property.getProperty("dataport.subscription_type", "New");
    
    if (pub_type == "New")
      {
        RTM_CRITICAL_SECTION_START;
        while ( g_publisher_new[ index ].tsk_id > 0 )
          {
            if ( g_publisher_new[ index ].active == 0 )
              {
                g_publisher_new[ index ].active = 1;
                break;
              }
            index++;
          }
        RTM_CRITICAL_SECTION_END;

        if ( g_publisher_new[ index ].tsk_id > 0 )
          {
            g_publisher_new[ index ].pub_Data = new PublisherNew( consumer,
                                                                  property, 
                                                                  g_publisher_new[ index ].tsk_id,
                                                                  g_publisher_new[ index ].sem_id1,
                                                                  g_publisher_new[ index ].sem_id2 );
            if ( g_publisher_new[ index ].pub_Data == NULL )
              {
                RTM_CRITICAL_SECTION_START;

                g_publisher_new[ index ].active = 0;

                RTM_CRITICAL_SECTION_END;
              }

            return g_publisher_new[ index ].pub_Data;
          }
      }
    else if (pub_type == "Periodic")
      {
        RTM_CRITICAL_SECTION_START;
        index = 0;
        while ( g_publisher_periodic[ index ].tsk_id > 0 )
          {
            if ( g_publisher_periodic[ index ].active == 0 )
              {
                g_publisher_periodic[ index ].active = 1;
                break;
              }
            index++;
          }
        RTM_CRITICAL_SECTION_END;

        if ( g_publisher_periodic[ index ].tsk_id > 0 )
          {
            g_publisher_periodic[ index ].pub_Data = new PublisherPeriodic( consumer,
                                                                            property, 
                                                                            g_publisher_periodic[ index ].tsk_id );
            if ( g_publisher_periodic[ index ].pub_Data == NULL )
              {
                RTM_CRITICAL_SECTION_START;

                g_publisher_periodic[ index ].active = 0;

                RTM_CRITICAL_SECTION_END;
              }

            return g_publisher_periodic[ index ].pub_Data;
          }
      }
    else if (pub_type == "Flush")
      {
	return new PublisherFlush(consumer, property);
      }
    return NULL;
  }
  
  /*!
   * @if jp
   * @brief Publisher˴
   * @else
   * @brief Destroy the Publishers
   * @endif
   */
  void PublisherFactory::destroy(PublisherBase* publisher)
  {
    int index = 0;
    while ( g_publisher_new[ index ].tsk_id > 0 )
      {
        if ( g_publisher_new[ index ].active != 0 &&
             g_publisher_new[ index ].pub_Data == publisher )
          {
            RTM_CRITICAL_SECTION_START;

            g_publisher_new[ index ].active   = 0;
            g_publisher_new[ index ].pub_Data = NULL;

            RTM_CRITICAL_SECTION_END;

            break;
          }
        index++;
      }

    index = 0;
    while ( g_publisher_periodic[ index ].tsk_id > 0 )
      {
        if ( g_publisher_periodic[ index ].active != 0 &&
             g_publisher_periodic[ index ].pub_Data == publisher )
          {
            RTM_CRITICAL_SECTION_START;

            g_publisher_periodic[ index ].active   = 0;
            g_publisher_periodic[ index ].pub_Data = NULL;

            RTM_CRITICAL_SECTION_END;

            break;
          }
        index++;
      }

    publisher->release();
    delete publisher;
  }
}; // namespace RTC
