// -*- C++ -*-
/*!
 * @file  PublisherNew.cpp
 * @brief PublisherNew class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherNew.cpp 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#include <rtm/PublisherNew.h>
#include <rtm/InPortConsumer.h>
#include <rtm/Properties.h>
#include <rtm/RTC.h>
#include <iostream>


namespace RTC
{
  /*!
   * @if jp
   * @brief 󥹥ȥ饯
   * @else
   * @brief Constructor
   * @endif
   */
  PublisherNew::PublisherNew(InPortConsumer* consumer,
			     const Properties& property,
                             ID tsk_id,
                             ID sem_id1,
                             ID sem_id2 )
    : m_consumer(consumer), m_running(true), m_data(), coil::Task(tsk_id)
  {
    m_data._mutex.init( sem_id1 );
    m_data._cond.init( sem_id2 );
    open(0);
  }

  /*!
   * @if jp
   * @brief ǥȥ饯
   * @else
   * @brief Destructor
   * @endif
   */
  PublisherNew::~PublisherNew()
  {
    delete m_consumer;
  }
  
  /*!
   * @if jp
   * @brief Observerؿ
   * @else
   * @brief Observer function
   * @endif
   */
  void PublisherNew::update()
  {
    if (m_data._mutex.trylock() != 0)
      {
	return;
      }
    m_data._updated = true;
    m_data._cond.signal();
    m_data._mutex.unlock();
	rot_rdq( TPRI_SELF );
    return;
  }
  
  /*!
   * @if jp
   * @brief PublisherNew::åɼ¹Դؿ
   * @else
   * @brief Thread execution function
   * @endif
   */
  int PublisherNew::svc(void)
  {
    while (m_running)
      {
	m_data._mutex.lock();
	
	// Waiting for new data updated
	while (!m_data._updated && m_running)
	  {
	    m_data._cond.wait();
	  }
	
	if (m_data._updated)
	  {
	    m_consumer->push(); 
	    m_data._updated = false;
	  }
	
	m_data._mutex.unlock();	
      }
    return 0;
  }

  
  /*!
   * @if jp
   * @brief 
   * @else
   * @brief Task start function
   * @endif
   */
  int PublisherNew::open(void *args)
  {
    m_running = true;
    this->activate();
    return 0;
  }
  
  
  /*!
   * @if jp
   * @brief λؿ
   * @else
   * @brief Task terminate function
   * @endif
   */
  void PublisherNew::release()
  {
    if (m_data._mutex.trylock() != 0)
      {
	return;
      }
    m_running = false;
    m_data._cond.signal(); //broadcast();
    m_data._mutex.unlock();
    wait();
  }
  
}; // namespace RTC
