/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
#include <RtORB/corba.hh>

CORBA_any::CORBA_any() : _type(NULL), _val(NULL), _release(1) 
{
}

CORBA_any::CORBA_any(const CORBA_any &o) : _type(NULL), _val(NULL), _release(1)
{
  _type = o._type;
  if (o._val) {
    copy_val(o._type, o._val);
  }
}

CORBA_any::~CORBA_any()
{
  free_();
}

void CORBA_any::free_()
{
  CORBA_any_clear(this);
}

void CORBA_any::copy_val(CORBA_TypeCode tc, CORBA_any_val *val)
{
  if (!tc) { return; }
  alloc(tc);

  switch(tc->kind) {
  case tk_char:
    _val->val_char = val->val_char; break;
  case tk_octet:
    _val->val_octet = val->val_octet; break;
  case tk_boolean:
    _val->val_bool = val->val_bool; break;
  case tk_ulong:
    _val->val_ulong = val->val_ulong; break;
  case tk_objref:
    if (val->val_obj) {
      _val->val_obj = CORBA_Object_dup(val->val_obj);
    }
    break;
  case tk_string:
    if (val->val_str) {
      _val->val_str = RtORB_strdup(val->val_str, "CORBA_any::copy_val");
    }
    break;
  case tk_except:
    _val->val_except = val->val_except; break;
  case tk_struct:
    {
      int len = val->val_encoded.len;
      char *data = val->val_encoded.data;
      _val->val_encoded.len = len;
      if (len > 0 &&  data != NULL) {
	memcpy(_val->val_encoded.data, data, len);
      }
    }

    break;
  default:
    ORB_ERROR_OUT2( "CORBA_any::copy_val : typecode %d not supported\n", (int)tc->kind );
    free_();
    break;
  }
  return;
}

CORBA_any & CORBA_any::operator=(const CORBA_any &o)
{
  free_();
  copy_val(o._type, o._val);
  return *this;
}

void CORBA_any::operator<<=(from_char from)
{
  alloc(TC_CORBA_char);

  _val->val_char = from.val;
}

CORBA_boolean CORBA_any::hasData(CORBA_TypeCode tc) const
{
  if (!_type) { return FALSE; }
  if (_type != tc) { return FALSE; }
  if (!_val) { return FALSE; }
  return TRUE;
}

CORBA_boolean CORBA_any::operator>>=(to_char to) const
{
  if (!hasData(TC_CORBA_char)) { return FALSE; }
  to.buf = _val->val_char;
  return TRUE;
}

void CORBA_any::alloc(CORBA_TypeCode tc)
{
  CORBA_any_alloc(this, tc, 1);
}

void CORBA_any::operator<<=(from_boolean from)
{
  alloc(TC_CORBA_boolean);
  _val->val_bool = from.val;
}

CORBA_boolean CORBA_any::operator>>=(to_boolean to) const
{
  if (!hasData(TC_CORBA_boolean)) { return FALSE; }
  to.buf = _val->val_bool;
  return TRUE;
}

void CORBA_any::operator<<=(from_octet from)
{
  alloc(TC_CORBA_octet);
  _val->val_octet = from.val;
}

CORBA_boolean CORBA_any::operator>>=(to_octet to) const
{
  if (!hasData(TC_CORBA_octet)) { return FALSE; }
  to.buf = _val->val_octet;
  return TRUE;
}

void CORBA_any::operator<<=(from_string from)
{
  alloc(TC_CORBA_string);
  _val->val_str = strdup(from.val);
}

CORBA_boolean CORBA_any::operator>>=(to_string to) const
{
  if (!hasData(TC_CORBA_string)) { return FALSE; }
  to.buf = _val->val_str;
  return TRUE;
}

CORBA_any::from_object::from_object(CORBA::Object_ptr &ptr_) : ptr(ptr_) 
{
  
}

void CORBA_any::operator<<=(from_object from)
{
  alloc(TC_CORBA_Object);
  
  CORBA_Object obj = from.ptr.impl();
  if (obj) {
    _val->val_obj = CORBA_Object_dup(obj);
  }
}

CORBA_boolean CORBA_any::operator>>=(to_object to) const
{
  if (!hasData(TC_CORBA_Object)) { return FALSE; }
  to.ptr = _val->val_obj;
  return TRUE;
}


void CORBA_any::operator<<=(CORBA_unsigned_long n)
{
  alloc(TC_CORBA_ulong);
  _val->val_ulong = n;
}


CORBA_boolean CORBA_any::operator>>=(CORBA_unsigned_long &n) const
{
  if (!hasData(TC_CORBA_ulong)) { return FALSE; }
  n = _val->val_ulong;
  return TRUE;
}

/*
  Any_var operators
 */

void CORBA_any::setData(struct CORBA_TypeCode_struct *tc, char *data, int len)
{
  CORBA_any_clear(this);
  CORBA_any_set_value(this, tc, data, len);
}

namespace CORBA {

  void Any_var::operator<<=(CORBA::ULong n)
  {
    if (!_ptr) {
      _ptr = new Any();
    }
    *_ptr <<= n;
  }
  void Any_var::operator<<=(const char *s)
  {
    if (!_ptr) {
      _ptr = new Any();
    }
    *_ptr <<= s;
  }

  void AnyObjectStream::from(CORBA::Any &any, CORBA_TypeCode tc, CORBA_Object obj)
  {
    any.alloc(tc);
  
    if (obj) {
      any._val->val_obj = CORBA_Object_dup(obj);
    }
  }
}
