/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
/*
 *
 *
 *
 */
#include <RtORB/corba.hh>
#include <assert.h>

CORBA_boolean CORBA_Object_struct::_is_equivalent(CORBA_Object_struct *other)
{
  if (!other) { return FALSE; }
  CORBA_Environment ev;
  memset(&ev, 0x00, sizeof(CORBA_Environment));
  return CORBA_Object_is_equivalent(this, other, &ev);
}

namespace CORBA{

  /* #############################################
   *
   * class CORBA::Object
   *
   * ############################################# */

  Object::Object(){
    _impl = 0;
    memset(&ev, 0, sizeof(CORBA_Environment));
  }

  Object::Object(CORBA_Object obj){
    _impl = obj;
    memset(&ev, 0, sizeof(CORBA_Environment));
  }

  Object & Object::operator=(const Object& other)
  {
    _impl = CORBA_Object_dup(other._impl);
    return *this;
  }

  Object::Object(const Object& other)
  {
    _impl = CORBA_Object_dup(other._impl);
    memcpy(&ev, &other.ev, sizeof(other.ev));
  }

  Object::~Object(){
	  /*
    if(_impl)
	if(CORBA_Object_free(_impl) < 1)
		_impl = 0;
	  */
  }

  Object_ptr Object::_duplicate(Object_ptr ptr)
  {
    if (!ptr) { return Object_ptr(); }
    CORBA_Object_dup(ptr.impl());
    return ptr;
	  
  }

  Boolean Object::_is_equivalent(Object_ptr other)
  {
    if (!_impl || !other) { return FALSE; }
    return CORBA_Object_is_equivalent(_impl, other.impl(), &ev);
  }

  CORBA_Object Object::release()
  {
    CORBA_Object tmp = _impl;
    _impl = NULL;
    return tmp;
  }

  /* #############################################
   *
   * class CORBA::Object_var
   *
   * ############################################# */
  Object_var::Object_var() : _obj(NULL)
  {
    COD_DUMP(("in Object_var()"));
  }
    
  Object_var::Object_var(const Var &o) : _obj(NULL) {
    COD_DUMP(("in Object_var(const Object_var &)"));

    /*
      CORBA_Object impl = o._impl;
      if (impl) {
      CORBA_Object_duplicate(impl, getEnv(impl));
      }
    */
    _obj = new Object(*_obj);
  }
    
  Object_var::Object_var(Object *p) : _obj(p)
  {
    COD_DUMP(("in Object_var(Object *)"));


    /*
      if (p) {
      CORBA_Object impl = p->impl();
      CORBA_Object_duplicate(impl, NULL);
      }
    */
  }

  CORBA_Environment * Object_var::getEnv(CORBA_Object impl)
  {
    /*
      if (!impl || !impl->impl_obj) { return NULL; }
      Object * obj = reinterpret_cast<Object*>(impl->impl_obj);
      return &obj->ev;
    */
    return NULL;
  }

  Object_var::Object_var(CORBA_Object impl) : _obj(NULL) {

    COD_DUMP(("in Object_var(CORBA_Object)"));
    /*
      if (_impl) {
      CORBA_Object_duplicate(_impl, getEnv(impl));
      }
    */
    set(impl);
  }

  void Object_var::set(CORBA_Object impl)
  {
    if (impl) {
      if (impl->impl_obj) {
	_obj = reinterpret_cast<Object*>(impl->impl_obj);
      } else {
	_obj = new Object(impl);
      }
    }
  }

  Object_var::Object_var(Object_ptr ptr) : _obj(ptr.object())
  {
    ;
  }
    
  Object_var::~Object_var() {
    if (_obj) {
      delete _obj; _obj = NULL;
    }
    /*
      if (_impl) {
      CORBA_Object_release(_impl, getEnv(_impl));
      }
    */
  }

  Object_var & Object_var::operator=(CORBA_Object impl) {
    if (_obj) {
      delete _obj; _obj = NULL;
    }
    set(impl);
    return *this;
  }

  Object_var & Object_var::operator=(Object_ptr ptr)
  {
    if (_obj) { delete _obj; _obj = NULL; }
    _obj = ptr.object();
    return *this;
  }

  Object_var::operator Object_ptr ()
  {
    return Object_ptr(_obj);
  }
    
  Object * Object_var::operator->()
  {
    return _obj;
  }

  bool Object_var::operator!() {
    return _obj == NULL? true: false;
  }

  Object * Object_var::object()
  {
    return _obj;
  }
  /* #############################################
   *
   * class CORBA::Object_ptr
   *
   * ############################################# */
  Object_ptr::Object_ptr(CORBA_Object impl) : _obj(NULL)
  {
    set(impl);
  }

  void Object_ptr::set(CORBA_Object impl)
  {
    if (impl) {
      if (impl->impl_obj) {
	_obj = reinterpret_cast<Object*>(impl->impl_obj);
      } else {
	_obj = new Object(impl);
      }
    }
  }


  Object_ptr::Object_ptr(void *p) : _obj(NULL)
  {
    assert(p == NULL);
  }

  CORBA_Object Object_ptr::impl()
  {
    return _obj? _obj->impl(): NULL;
  }

  /* #############################################
   *
   * functions
   *
   * ############################################# */
  
  CORBA_string string_dup(const char* str)
  {
    if (!str) { return NULL; }
    return RtORB__strdup(str, "CORBA::string_dup");
  }
  
  CORBA_string string_alloc(CORBA_unsigned_long n)
  {
    if (n <= 0) { return NULL; }
    return (CORBA_string)RtORB__alloc(n+1, "CORBA::string_alloc");
  }
};

CORBA_string_var::~CORBA_string_var()
{
  if (_str) {
    RtORB__free(_str, "~CORBA_string_var");
  }
}
  

char * CORBA_string_var::alloc(const char *p) {
  if (p) {
    _str = RtORB__strdup(p, "CORBA_string_var::alloc");
  }
  return _str;
}
