/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
#include <RtORB/corba.hh>

namespace CORBA
{
   ORB_ptr ORB_init(int& argc, char **argv){
     ORB_ptr orb =  new ORB();
     CORBA_ORB impl = CORBA_ORB_init(&argc, argv, 0, &orb->ev);
     if(impl) orb->__set_impl(impl);
     return orb;
   }

   
//// Class ORB
  ORB::ORB(){
    _impl = NULL;
    memset(&ev, 0, sizeof(CORBA_Environment));
  }

  ORB::ORB(CORBA_ORB orb){
    _impl = orb;
    memset(&ev, 0, sizeof(CORBA_Environment));
  }

  ORB::~ORB(){
    if (_impl) {
      CORBA_ORB_shutdown(_impl, 0,  &ev);
      CORBA_ORB_destroy(_impl, &ev);
      _impl = NULL;
    }
  }

  void ORB::__set_impl(CORBA_ORB orb){
    _impl = orb;
  }


  Object_ptr ORB::resolve_initial_references(char *name) throw (Exception) {
    CORBA_Object obj;
    if(!_impl) return Object_ptr();

    obj = CORBA_ORB_resolve_initial_references(_impl, name, &ev);
    catchAndThrowDefaultException(&ev);

    if (obj) { return Object_ptr(new Object(obj)); }

    ORB_ERROR_OUT1( "Error in resolve_initial_references.\n" );
    return Object_ptr();
  }

  boolean ORB::__is_nil(){
    if(_impl == NULL) return TRUE;
    return FALSE;
  }

  String_ptr ORB::object_to_string(Object_ptr obj) throw (Exception) {
    unsigned char * str = CORBA_ORB_object_to_string(_impl,  obj.impl(), &ev);
    catchAndThrowDefaultException(&ev);
    return (String_ptr)str;
  }

  Object_ptr ORB::string_to_object(const char* str) throw (Exception)
  {
    char *p = const_cast<char*>(str);
    CORBA_Object obj = CORBA_ORB_string_to_object(_impl, (unsigned char*)str, &ev);
    catchAndThrowDefaultException(&ev);
    return Object_ptr(obj);
  }

  void ORB::run() {
    CORBA_ORB_run(_impl, &ev);
    return;
  }

  CORBA::Boolean ORB::work_pending()
  {
    return CORBA_ORB_work_pending(_impl, &ev);
  }
  
  void ORB::perform_work()
  {
    return CORBA_ORB_perform_work(_impl, &ev);
  }

  /*
  static Ptr _duplicate(Ptr)
  {
  }
  */
  ORB::Ptr ORB::_nil()
  {
    return NULL;
  }
  

  void ORB::shutdown(boolean flag){
    CORBA_ORB_shutdown(_impl, flag,  &ev);
    return;
  }

  void ORB::destroy(){
    if (_impl) {
      CORBA_ORB_destroy(_impl, &ev);
      _impl = NULL;
    }
    return;
  }

  /* ##########################
   *
   *  ORB_var
   *
   * ########################## */

  ORB_var::~ORB_var() {
    if (_ptr) {
      delete _ptr; _ptr = NULL;
    }
  }

  ORB_var & ORB_var::operator=(ORB *ptr) {
    if (_ptr) {
      delete _ptr; _ptr = NULL;
    }
    _ptr = ptr;
    return *this;
  }
}
