/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
/*
 *  RtORB Allocater
 *
 *
 */

#define _GNU_SOURCE
#include <string.h>
#include <RtORB/corba.h>

////////// allocater
void *
RtORB__alloc(long size, const char *info){
  void *retval = (void *)malloc(size);
	ORB_MESSAGE_OUT3("malloc in %s: (0x%x)\n", info, (int)retval);
#if DEBUG
	ORB_ERROR_OUT3("%s: alloc size = %d\n", info, (int)size);
#endif

  return retval;
}

void *
RtORB__calloc(long size, long n, const char *info){
  void *retval = (void *)calloc(size, n);
	ORB_MESSAGE_OUT3("malloc in %s: (0x%x)\n", info, (int)retval);
#if DEBUG
	ORB_ERROR_OUT3("%s: alloc size = %d\n", info, (int)size);
#endif

  return retval;
}

void *
RtORB__realloc(void *ptr, long size, const char *info){
  void *retval = (void *)realloc(ptr, size);
	ORB_MESSAGE_OUT3("realloc in %s: (0x%x)\n", info, (int)retval);
#if DEBUG
	ORB_ERROR_OUT3("%s: alloc size = %d\n", info, (int)size);
#endif

  return retval;
}

void *
RtORB_alloc_by_typecode(CORBA_TypeCode tc, int len, const char *info){
  SKIP_ALIAS(tc);
  switch(tc->kind){
    case tk_null:
    case tk_void:
      return NULL;
/*
    case tk_sequence:
      return RtORB_alloc(sizeof(CORBA_SequenceBase) *len , info);
*/
    default:
      return (void *)RtORB_alloc(size_of_typecode(tc) * len, info);
/*
      if (tc->size) return RtORB_alloc(tc->size * len, info);
      else return RtORB_alloc(sizeof(void *) * len, info);
*/
  }
}

void *
RtORB_realloc_by_typecode(void *ptr, CORBA_TypeCode tc, int len, const char *info){
  SKIP_ALIAS(tc);
  switch(tc->kind){
    case tk_null:
    case tk_void:
      return NULL;
/*
    case tk_sequence:
      return RtORB_alloc(sizeof(CORBA_SequenceBase) *len , info);
*/
    default:
      return (void *)RtORB__realloc(ptr, size_of_typecode(tc) * len, info);
/*
      if (tc->size) return RtORB_alloc(tc->size * len, info);
      else return RtORB_alloc(sizeof(void *) * len, info);
*/
  }
}

void 
RtORB__free(void *val, const char *info){
#if DEBUG_MALLOC
	ORB_MESSAGE_OUT3("FREE in %s: (0x%x)\n", info, (int)val);
#endif
   free(val);
  return;
}

void 
RtORB_free_by_typecode(CORBA_TypeCode tc, void *val, int flag){
   if (!val) return;
   SKIP_ALIAS(tc);
#if 0
	ORB_MESSAGE_OUT2("Typecode = %d\n", (int)tc->kind);
#endif
   switch(tc->kind){
      case tk_null:
      case tk_void:
	 return;

      case tk_octet:
      case tk_char:
      case tk_boolean:
	 break;

      case tk_short:
      case tk_ushort:
	 break;
	
      case tk_long:
      case tk_ulong:
	 break;

      case tk_float:
	 break;

      case tk_double:
	 break;

      case tk_string:
         RtORB_free(*(char **)val, "RtORB_free_by_typecode(string)");
	 break;

      case tk_sequence:
	 {
	   int i;
           CORBA_SequenceBase *sb = (CORBA_SequenceBase *)val;
	   void **buf = (void **)sb->_buffer;
	   int len = size_of_typecode(tc->member_type[0]);

	   for(i=0;i<sb->_maximum;i++){
             //RtORB_free_by_typecode(tc->member_type[0], &sb->_buffer[i], 0);
             RtORB_free_by_typecode(tc->member_type[0], buf, 0);
	     buf = (void **)((char *)buf + len);
	   }
           RtORB_free(sb->_buffer, "RtORB_free_by_typecode(sequence)");
	 }
	 break;

      case tk_except:
      case tk_struct:
	 {
	   int i;
	   void *tmp;
	   tmp = val;
	   
	   for(i=0;i< tc->member_count;i++){
             RtORB_free_by_typecode(tc->member_type[i], tmp, 0);
	     tmp = (char *)tmp + size_of_typecode(tc->member_type[i]);
	   }
	 }
	 break;
      case tk_objref:
	 {
	    CORBA_Object obj = *(CORBA_Object *)val;
	    if(!obj) { break;}
#if DEBUG
		ORB_ERROR_OUT3("CORBA_Object_free: 0x%x, ref=%d\n", (int)obj, (int)obj->ref);
#endif
	    CORBA_Object_free(obj);
	 }
	 break;
      case tk_any:
	{
	  CORBA_any *any = (CORBA_any*)val;
	  if (!any) { break; }
	  CORBA_any_clear(any);
	  break;
	}
      default: 
	 break;
  }
  if(flag) RtORB_free(val, "RtORB_free_by_typecode");
  return;
}


/*
 *  Free_Arguments2:
 *    free  memory chunk allocated by 'deMarshal_Arg2'
 *    This function called from 'invokeServant2'.
 *
 */

void 
RtORB_Arguments_free(void **argv, int in_argc, CORBA_IArg *in_argv){
  int i;

  for(i=0;i<in_argc;i++){
    if(in_argv[i].io == CORBA_I_ARG_OUT || in_argv[i].io == CORBA_I_ARG_INOUT){
      /*
      RtORB_free_by_typecode(in_argv[i].tc, *(void **)argv[i], 1);
      RtORB_free(argv[i], "RtORB_Arguments_free");
      */
    }else{
        RtORB_free_by_typecode(in_argv[i].tc, argv[i], 1);
    }
  }
  
  return;
}

void
RtORB_Result__free(CORBA_TypeCode tc, void **result){
  SKIP_ALIAS(tc);
  switch(tc->kind){
  case tk_null:
  case tk_void:
    return;
  default:
    break;
  }
  if (CORBA_TypeCode_is_fixed_size(tc)) {
    RtORB_free_by_typecode(tc, (void*)result, 1);
  } else {
    RtORB_free_by_typecode(tc, result[0], 1);
    RtORB__free(result, "RtORB_Result_free");
  }
  return;
}

//////////////////////////////////
/*
 * void *
 * RtORB_typecode_alloc(CORBA_TypeCode)
 *
 */
void *
RtORB_typecode_alloc(CORBA_TypeCode tc){
  return (void *)RtORB_calloc(size_of_typecode(tc), 1, "");
}
/*
 * void
 * RtORB__freekids(CORBA_TypeCode, void *, void *)
 *
 */
void
RtORB__freekids(CORBA_TypeCode tc, void *p, void *d){
  RtORB_free_by_typecode(tc, p, 1);
  return;
}

/*
 * void *
 * RtORB__allocbuf(CORBA_TypeCode, long )
 *
 */
void *
RtORB__allocbuf(CORBA_TypeCode tc, long length){
  return RtORB_alloc_by_typecode(tc->member_type[0], length, "");
}

void *
RtORB__reallocbuf(void *ptr, CORBA_TypeCode tc, long length){
  return RtORB_realloc_by_typecode(ptr, tc->member_type[0], length, "");
}

/*
 * void *
 * RtORB_sequence_allocbuf(CORBA_TypeCode, long )
 *
 */
void *
RtORB_sequence_allocbuf(CORBA_TypeCode tc, long length){
  return RtORB_alloc_by_typecode(tc->member_type[0], length, "");
}


//// Strdup

char * RtORB__strdup(const char *str, const char *info){
   char *retval;
   retval = strdup(str);
#if DEBUG
	ORB_MESSAGE_OUT3("strdup in %s: (0x%x)\n", info, (int)retval);
#endif
   return retval;

}

char * RtORB__strndup(const char *str, int n, const char *info){
   char *retval;
   retval = strndup(str, n);
	ORB_MESSAGE_OUT3("strndup in %s: (0x%x)\n", info, (int)retval);
   return retval;

}
