/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
/*
 *  RtORB:
 *   IIOP-GIOP
 *
 *  Copyright(C) 2006, Isao Hara, AIST
 *
 *  $Revision: 1.9 $
 *  $Date: 2008/06/04 15:10:23 $
 *  $Id: giop.c,v 1.9 2008/06/04 15:10:23 yoshi Exp $
 *
 */

#include <RtORB/corba.h>
#include <RtORB/giop.h>
#include <RtORB/hashtable.h>
#include <RtORB/sockport.h>
#include <net/net.h>

#ifdef USE_SHMC
#include <sys/types.h>
#include <sys/stat.h>
#ifndef SHMC_TMPDIR
#define SHMC_TMPDIR "/tmp/shmc"
#endif
#endif


/////
GIOP_Connection *
GIOP_Connection__create(){
  GIOP_Connection *conn=(GIOP_Connection *)RtORB_alloc(sizeof(GIOP_Connection),
		 "GIOP_Connection__alloc" );
  memset(conn, 0, sizeof(GIOP_Connection));

  conn->hostname = (char *)Get_IP_Address();
  conn->activate = FALSE;
//  conn->type = 0;
//  conn->parent = NULL;

#ifdef USE_SHMC
  conn->shmc = NULL;
#endif

#if USE_THREAD
  conn->thread = (pthread_t)NULL;
#endif

  return conn;
}


void
GIOP_Connection__open(GIOP_Connection *conn, short port){
  if(!conn) return;

  make_server_connection(conn, port);
  
  return;
}

void
GIOP_Connection__shutdown(GIOP_Connection *conn){
#if USE_THREAD
  void *result; 
#endif
  if(!conn) return;
  if(conn->activate){ conn->activate = FALSE; }
#if USE_THREAD
  /// wait for shutdown thread
  pthread_join(conn->thread, &result);
#endif
  return;
}

void
GIOP_Connection__close(GIOP_Connection *conn){
  if(!conn) return;
  if(conn->activate) GIOP_Connection__shutdown(conn);
  if(conn->sock) close(conn->sock);
#ifdef USE_SHMC
  if(conn->shmc) { shmcClose(conn->shmc); conn->shmc = NULL; }
#endif

  conn->port = 0;
  conn->sock = 0;
  
  return;
}

/* For multi thread */
void *
GIOP_Connection__run(GIOP_Connection *conn){
  if(!conn) return NULL;
//  if(conn->type != 0) return;
  if(conn->sock && conn->activate == FALSE){
   	 ORB_ERROR_OUT1("run GIOP_Connection\n");
     conn->activate = TRUE;
     server_socket_loop(conn->sock, (float)1000.0, NULL, NULL, RecvMessage);
  }
  return NULL;
}

void
GIOP_Connection__free(GIOP_Connection *conn){
  if(!conn) return;
  if(conn->sock) GIOP_Connection__close(conn);
#ifdef USE_SHMC
  if(conn->shmc) {
    shmcClose(conn->shmc);
  }
#endif

  if(conn->hostname){
    RtORB_free(conn->hostname, "GIOP_Connection__free(hostname)");
  }
  RtORB_free(conn, "GIOP_Connection__free");
}


//////////////////////////////////////////////////////////////
GIOP_MessageHeader *GIOP_Create_MessageHeader(int order, int version){
  GIOP_MessageHeader *header = (GIOP_MessageHeader *)RtORB_alloc(SIZEOF_GIOP_HEADER, "GIOP_Create_MessageHeader");

  memcpy(header->magic, "GIOP", 4);
  header->version.major = 1;
  header->version.minor = version;
  header->flags = order;

  return header;
}

////////////////////////////////////////////////////////////
/***
 *
 */
int recieveMessage(GIOP_ConnectionHandler *h, GIOP_MessageHeader *header, octet *buf, int maxMsg){

  if (GIOP_ConnectionHandler_recv(h, buf, SIZEOF_GIOP_HEADER) < 0) {
#if DEBUG
	ORB_ERROR_OUT1("Bad GIOP message\n");
#endif
    return -1;
  }
  memcpy(header, buf, SIZEOF_GIOP_HEADER);
  if((header->flags & 0x01) == 0 ) header->message_size=ntohl(header->message_size);

  if(header->message_size > maxMsg){
	ORB_ERROR_OUT2("Buffer overflow(size = %#x)!!\n", header->message_size);
    return -1;
  }

  if (GIOP_ConnectionHandler_recv(h, buf+SIZEOF_GIOP_HEADER, header->message_size) < 0) {
	ORB_ERROR_OUT1("Bad Reply message\n");
    return -1;
  }
  return header->message_size;
}


///////////////

int reply_Message(GIOP_ConnectionHandler *h, GIOP_RequestHeader *request_header,
		GIOP_ReplyBody *reply_body,
		int version)
{
//unsigned char buf[MaxMessageSize];
  unsigned char *buf = NULL;
  int len=0;
  GIOP_ReplyHeader header;

  buf = (unsigned char *)malloc( MaxMessageSize );

  memset(buf, 0, MaxMessageSize);

  if (version < 2){
    header._1_0.service_context.num = 0;
    header._1_0.request_id = request_header->_1_0.request_id;
    header._1_0.reply_status = reply_body->status;
  }else{
    header._1_2.request_id = request_header->_1_2.request_id;
    header._1_2.reply_status = reply_body->status;
    header._1_2.service_context.num = 0; /// Not Implemented for service_context
  }

  switch(reply_body->status){
    case GIOP_SYSTEM_EXCEPTION:
         {
           int id_len = reply_body->exception.exception_id._length +1;
           int exception_size = id_len+16;
           int current=0;
           octet ebuf[exception_size];

           marshalLong(ebuf, &current, reply_body->status);
           marshal_Octet_Sequence(ebuf, &current, 
                          reply_body->exception.exception_id._buffer, id_len);
           marshalLong(ebuf, &current, reply_body->exception.minor_code_value);
           marshalLong(ebuf, &current, reply_body->exception.completion_status);

           MarshalReply(buf, &len, &header, ebuf, exception_size, version);
	 }
	 break;
    case GIOP_USER_EXCEPTION:
	 ORB_ERROR_OUT1("User Exception!!!\n");
    case GIOP_NO_EXCEPTION:
         MarshalReply(buf, &len, &header, reply_body->body,
			 reply_body->body_size, version);
	 break;
  }

#if DEBUG
   ORB_ERROR_OUT2("===== GIOP Reply (%d) =====\n", len);
   dumpMessage(buf, len);
#endif

 /// Send Message
  if (GIOP_ConnectionHandler_send(h, buf, len) < 0){
    if ( buf ) {
      free( buf );
    }
    return -1;
  }
  if ( buf ) {
    free( buf );
  }
  return 1;
}


/*
 * invoke
 *
 */
GIOP_ReplyBody *invokeServant(PortableServer_POA poa,
		GIOP_RequestHeader *header,
  		CORBA_Environment *env,
		octet *arg_buf, int version, int order)
{
  char *function;
  char *obj_key;
  RtORB_POA_Object *poa_obj;
  PortableServer_ClassInfo *info;
  CORBA_Class_Method *m_data;
  void *(*impl_method)();
  void (*call_impl_func)();
  void * *argv;
  void * result;

  void RtORB_Result__free(CORBA_TypeCode tc, void **result);

  hashtable *tbl; 

  GIOP_ReplyBody *reply = (GIOP_ReplyBody *)RtORB_alloc(sizeof(GIOP_ReplyBody),
		  "ReplyBody in invokeServant");

  reply->status = GIOP_NO_EXCEPTION ;
  reply->body = NULL;
  reply->body_size = 0;

  if(poa) tbl = poa->object_map;
  else tbl = _ORB_->_object_table;
  
  if (version > 1){
     obj_key = header->_1_2.target.object_key._buffer;
     function = header->_1_2.operation._buffer;
  }else{
     obj_key = header->_1_0.object_key._buffer;
     function = header->_1_0.operation._buffer;
  }
  if(!obj_key) obj_key = "NameService";
  poa_obj = (RtORB_POA_Object *)getValue(tbl, obj_key);

#if DEBUG
    if (version > 1){
	  ORB_ERROR_OUT3("POA Object %s, function: %s\n", header->_1_2.target.object_key._buffer, function);
    }else{
	  ORB_ERROR_OUT3("POA Object %s, function: %s\n", header->_1_0.object_key._buffer, function);
    }
#endif

  if(poa_obj == NULL){
    reply->status = GIOP_SYSTEM_EXCEPTION;
    reply->exception.exception_id._buffer= "Unknown Object";
    reply->exception.exception_id._length= 14;
    reply->exception.minor_code_value = 0;
    reply->exception.completion_status = 0;
    return reply;

  }else{
    info = (PortableServer_ClassInfo *)poa_obj->_private;
    PortableServer_ServantBase *sb = (PortableServer_ServantBase*)poa_obj->servant;

    call_impl_func = (*info->impl_finder)(&sb->_private, function, &m_data,
		    &impl_method );

    if (!call_impl_func){
      if(!strcmp(function, "_is_a") && sb->vepv && sb->vepv[0]->is_a ){
	  int pos, len;
	  pos = 0;
          char *id = deMarshalString(arg_buf, &pos, order, &len);
          CORBA_boolean retval = (sb->vepv[0]->is_a)(&sb->_private, id , env );
          RtORB__free(id, "_is_a");
	  reply->body_size = 1;
          reply->body = (char *) RtORB_alloc(reply->body_size,
			   "Reply->body(_is_a) in invokeServant");
	   memcpy(reply->body, &retval , 1);
	   return reply;
      }else if(!strcmp(function, "_non_existent") && sb->vepv && sb->vepv[0]->non_existent ){
	  int pos, len;
	  pos = 0;
          CORBA_boolean retval = (sb->vepv[0]->non_existent)(&sb->_private, env );
	  reply->body_size = 1;
          reply->body = (char *) RtORB_alloc(reply->body_size,
			   "Reply->body(_is_a) in invokeServant");
	   memcpy(reply->body, &retval , 1);
	   return reply;
      }else{
		ORB_ERROR_OUT2("No such a Function: %s\n", function);
        reply->status = GIOP_SYSTEM_EXCEPTION;
        reply->exception.exception_id._buffer= "Unknown Function";
        reply->exception.exception_id._length= 14;
        reply->exception.minor_code_value = 0;
        reply->exception.completion_status = 0;
        return reply;
      }
    }
    argv = (void * *)deMarshal_Arg(arg_buf, m_data->in_argc,
		    m_data->in_argv, order);
    result = (void *) Result_alloc(m_data->retval);
    (*call_impl_func)(sb, result, m_data, argv, env, impl_method);

    switch(env->_major){
      case CORBA_NO_EXCEPTION:
    	Marshal_Reply_Arguments(reply, result, argv, m_data);
	break;
      case CORBA_USER_EXCEPTION:
	{
// 	   char reply_buf[MaxMessageSize];
           char *reply_buf = NULL;
	   int size = 0;

           reply_buf = (char * )malloc( MaxMessageSize );

	   ORB_ERROR_OUT3("User exception(%s): %s\n", function, env->_repo_id);
           marshalString(reply_buf, &size, env->_repo_id);
           marshal_by_typecode(reply_buf, &env->_params, 
			    tk_any, &size);
	   reply->body_size = size;
           reply->body = (char *) RtORB_alloc(reply->body_size,
			   "Reply->body(USER_EXCEPTION) in invokeServant");
	   memcpy(reply->body, reply_buf, size);

           reply->status = GIOP_USER_EXCEPTION;

           if ( reply_buf ){
              free( reply_buf );
           }
	}
	break;
      case CORBA_SYSTEM_EXCEPTION:
		ORB_ERROR_OUT3("System exception occured in %s: %s\n", function, env->_repo_id);
        reply->status = GIOP_SYSTEM_EXCEPTION;
        reply->exception.exception_id._buffer=env->_repo_id;
        reply->exception.exception_id._length=strlen(env->_repo_id);
        reply->exception.minor_code_value = 0;
        reply->exception.completion_status = 0;
	break;
      default:
	break;
    }

    // 080327 free later
    RtORB_Arguments_free(argv, m_data->in_argc, m_data->in_argv);

    //RtORB_Result__free(m_data->retval, result);
    RtORB_free(argv, "deMarshal_Arg in invokeServant");
  }

  return reply;
}

int find_object(PortableServer_POA poa, char *key, char *ior){
  hashtable *tbl;

  if(poa) tbl = poa->object_map;
  else tbl = _ORB_->_object_table;

  if ( getValue(tbl, key) == NULL) return  GIOP_UNKNOWN_OBJECT;
  return  GIOP_OBJECT_HERE;
}

int reply_locateMessage(PortableServer_POA poa, GIOP_ConnectionHandler *h,
		GIOP_LocateRequestHeader *locate_request_header, int version)
{
//unsigned char buf[MaxMessageSize];
  unsigned char *buf = NULL;
  int len=0;
  char *object_key;
  char *ior = NULL;  /// Not use
  
  buf = (unsigned char *)malloc( MaxMessageSize );
  
  GIOP_LocateReplyHeader header;
  if (version < 2){
    header.request_id = locate_request_header->_1_0.request_id;
    object_key = locate_request_header->_1_0.object_key._buffer;
  }else{
    header.request_id = locate_request_header->_1_2.request_id;
    object_key = locate_request_header->_1_2.target.object_key._buffer;
  }

  header.locate_status = find_object(poa, object_key, ior);
  MarshalLocateReply(buf, &len, &header, ior, version);

  if (GIOP_ConnectionHandler_send(h, buf, len) < 0){
    if ( buf ) {
      free( buf );
    }
    return -1;
  }
  if ( buf ) {
    free( buf );
  }
  return 1;
}

//////////////////////////////////////////////////////////////////////
// Create Message
//
unsigned long createRequest(octet *buf, int response, 
		char *object_key, int object_key_len,
		char *operation, int operation_len,
		octet *args, int args_len, 
		int version)
{
  int len;

  GIOP_RequestHeader *header = (GIOP_RequestHeader *)newRequestHeader();

  memset(buf, 0, MaxMessageSize);

  GIOP_MessageHeader *MsgHeader = GIOP_Create_MessageHeader(RTORB_BYTE_ORDER, version);
  MsgHeader->message_type = GIOP_Request ;

  if(version < 2){
   header->_1_0.request_id = next_request_id();
   header->_1_0.response_expected = response;
   set_CORBA_Sequence_Octet(&header->_1_0.object_key, object_key, object_key_len);
   set_CORBA_Sequence_Octet(&header->_1_0.operation, operation, operation_len);
  }else{
   header->_1_2.request_id = next_request_id();
   header->_1_2.response_flags = response;
   header->_1_2.target_type = GIOP_KeyAddr;
   set_CORBA_Sequence_Octet(&header->_1_2.target.object_key, object_key, object_key_len);
   set_CORBA_Sequence_Octet(&header->_1_2.operation, operation, operation_len);
  }
  len = MarshalRequest(buf, header, args, args_len, version);

  MsgHeader->message_size = len - SIZEOF_GIOP_HEADER;
  memcpy(buf, MsgHeader, SIZEOF_GIOP_HEADER);

  deleteRequestHeader(header, version);
  RtORB_free(MsgHeader, "createRequest");

  return len;
}

/*
 * Cancel Request
 */
unsigned long createCancelRequest(octet *buf, unsigned long request_id)
{
  unsigned long version = 2;
  int current = SIZEOF_GIOP_HEADER;

  memset(buf, 0, MaxMessageSize);

  GIOP_MessageHeader *MsgHeader = GIOP_Create_MessageHeader(RTORB_BYTE_ORDER, version);
  MsgHeader->message_type = GIOP_CancelRequest ;
  MsgHeader->message_size = 4;
  memcpy(buf, MsgHeader, SIZEOF_GIOP_HEADER);

  marshalLong(buf, &current, request_id);
  RtORB_free(MsgHeader,"createCancelRequest");

  return SIZEOF_GIOP_HEADER + 4; 
}


int createLocateRequest(octet *buf, char *object_key, int len){
  int size;
  int version = 2;
  GIOP_LocateRequestHeader Header;

  memset(buf, 0, MaxMessageSize);

  GIOP_MessageHeader *MsgHeader = GIOP_Create_MessageHeader(RTORB_BYTE_ORDER, version);
  MsgHeader->message_type = GIOP_LocateRequest ;

  if (version < 2){
    Header._1_0.request_id = next_request_id();
    Header._1_0.object_key._length = len;
    Header._1_0.object_key._buffer = object_key;
  }else{
    Header._1_2.request_id = next_request_id();
    Header._1_2.target_type = GIOP_KeyAddr;
    Header._1_2.target.object_key._length = len;
    Header._1_2.target.object_key._buffer = object_key;
  }

  size = MarshalLocateRequest(buf, &Header, version);
  if (size < 0) return -1;

  MsgHeader->message_size = size - SIZEOF_GIOP_HEADER ;
  memcpy(buf, MsgHeader, SIZEOF_GIOP_HEADER);

  RtORB_free(MsgHeader, "createLocateRequest");
  return size ;
}

/*
 * CloseConnection Message
 */
unsigned long createCloseConnectionMessage(octet *buf)
{
  unsigned long version = 2;

  memset(buf, 0, MaxMessageSize);

  GIOP_MessageHeader *MsgHeader = GIOP_Create_MessageHeader(RTORB_BYTE_ORDER, version);
  MsgHeader->message_type = GIOP_CloseConnection ;
  MsgHeader->message_size = 0;
  memcpy(buf, MsgHeader, SIZEOF_GIOP_HEADER);

  RtORB_free(MsgHeader,"createCloseConnectionRequest");
  return SIZEOF_GIOP_HEADER; 
}

/*
 * MessageError Message
 */
unsigned long createMessageErrorMessage(octet *buf){
  unsigned long version = 0;

  memset(buf, 0, MaxMessageSize);

  GIOP_MessageHeader *MsgHeader = GIOP_Create_MessageHeader(RTORB_BYTE_ORDER, version);
  MsgHeader->message_type = GIOP_MessageError ;
  MsgHeader->message_size = 0;
  memcpy(buf, MsgHeader, SIZEOF_GIOP_HEADER);

  RtORB_free(MsgHeader,"createMessageErrorMessage");
  return SIZEOF_GIOP_HEADER; 
}

/*
 *
 *
 */
int requestLocation(GIOP_ConnectionHandler *h, CORBA_URL *ior){
//char buf[MaxMessageSize];
  char *buf = NULL;
  int len;
  int ret;

  buf = (char *)malloc( MaxMessageSize );

  len = createLocateRequest(buf, ior->object_key, ior->object_key_len);

  ret = GIOP_ConnectionHandler_send(h, buf, len);
  if ( buf ) {
    free( buf );
  }
  return ret;
}
/*
 *
 * confirm Location
 */
int confirmLocation(GIOP_ConnectionHandler *h, CORBA_URL *ior){
//char buf[MaxMessageSize];
  char *buf;
  int len;
  GIOP_MessageHeader header;
  GIOP_LocateReplyHeader *locate_reply_header;
  int result;
  CORBA_Sequence_Octet *body;

  buf = (char *)malloc( MaxMessageSize );

  len = createLocateRequest(buf, ior->object_key, ior->object_key_len);
  GIOP_ConnectionHandler_send(h, buf, len);

  if(recieveMessage(h, &header, buf, MaxMessageSize) < 0){
    if ( buf ) {
      free( buf );
    }
    return (-1);
  }

  body = (CORBA_Sequence_Octet *)new_CORBA_Sequence_Octet(0);
  locate_reply_header = (GIOP_LocateReplyHeader *)newLocateReplyHeader();
  deMarshalLocateReply(locate_reply_header, body, buf, &header);
  result =locate_reply_header->locate_status;
  delete_CORBA_Sequence_Octet(body, 1);
  RtORB_free(locate_reply_header,"confirmLocation");

  if ( buf ) {
    free( buf );
  }
  return result;
}

/*
 *
 *  GIOP Request Procedure
 */
void GIOP_Request_perform(GIOP_ConnectionHandler *h, octet *buf, PortableServer_POA poa,
  GIOP_MessageHeader *header, CORBA_Sequence_Octet *body,
  CORBA_Environment *env)
{
  GIOP_RequestHeader *request_header;
  GIOP_ReplyBody *reply_body;
  int byte_order = 0;

  if((header->flags & 0x01) != 0 ) byte_order = 1;

#ifdef DEBUG
  ORB_ERROR_OUT1("==== GIOP Message ====\n");
  dumpMessage(buf, header->message_size + SIZEOF_GIOP_HEADER);
#endif
  request_header = (GIOP_RequestHeader *)newRequestHeader();
  deMarshalRequest(request_header, body, buf, header);
  reply_body = invokeServant(poa, request_header, env, body->_buffer,
		  header->version.minor, byte_order);
  reply_Message(h, request_header, reply_body, header->version.minor);
  deleteReplyBody(reply_body);
  deleteRequestHeader(request_header, header->version.minor);
  return;
}

/*
 * GIOP Reply 
 *
 */
void GIOP_Reply_perform(GIOP_ConnectionHandler *h, octet *buf, PortableServer_POA poa,
  	GIOP_MessageHeader *header, CORBA_Sequence_Octet *body,
	CORBA_Environment *env)
{
  GIOP_ReplyHeader *reply_header;
  int reply_status;
   
   reply_header = (GIOP_ReplyHeader *)newReplyHeader();
   deMarshalReply(reply_header, body, buf, header);

   if(header->version.minor < 2){
     reply_status = reply_header->_1_0.reply_status;
   }else{
     reply_status = reply_header->_1_2.reply_status;
   }

   switch(reply_status){
     case GIOP_NO_EXCEPTION:
       break;
     case GIOP_USER_EXCEPTION:
	   ORB_ERROR_OUT1("User Exception occured!!!\n");
       break;
     case GIOP_SYSTEM_EXCEPTION:
	   ORB_ERROR_OUT1("System Exception occured\n");
       break;
     case GIOP_LOCATION_FORWARD:
     case GIOP_LOCATION_FORWARD_PERM:
       dumpMessage(buf + SIZEOF_GIOP_HEADER, header->message_size);
	   ORB_ERROR_OUT1("Location forward\n");
       break;
     case GIOP_NEEDS_ADDRESSING_MODE:
     default:
       dumpMessage(buf + SIZEOF_GIOP_HEADER, header->message_size);
     break;
   }
   deleteReplyHeader(reply_header, header->version.minor);
   
   return;
}

/*
 * GIOP Cancel Request 
 *
 */
void GIOP_CancelRequest_perform(GIOP_ConnectionHandler *h, octet *buf, PortableServer_POA poa,
  	GIOP_MessageHeader *header, CORBA_Sequence_Octet *body,
	CORBA_Environment *env)
{
  GIOP_CancelRequestHeader *cancel_request_header;

  cancel_request_header = (GIOP_CancelRequestHeader *)newCancelRequestHeader();
  deMarshalCancelRequest(cancel_request_header, buf, header);
  ORB_ERROR_OUT2("Call Cancel %d\n", (int)cancel_request_header->request_id);
  RtORB_free(cancel_request_header, "GIOP_CancelRequest_perform");
  return;
}

/*
 * GIOP Location Request 
 *
 */
void GIOP_LocationRequest_perform(GIOP_ConnectionHandler *h, octet *buf, PortableServer_POA poa,
  	GIOP_MessageHeader *header, CORBA_Sequence_Octet *body,
	CORBA_Environment *env)
{
  GIOP_LocateRequestHeader *locate_request_header;

  locate_request_header = (GIOP_LocateRequestHeader *)newLocateRequestHeader();
  deMarshalLocateRequest(locate_request_header, buf, header);
  reply_locateMessage(poa, h, locate_request_header, header->version.minor);

  deleteLocateRequestHeader(locate_request_header, header->version.minor);
  return;
}

/*
 * GIOP Location Reply 
 *
 */
void GIOP_LocationReply_perform(GIOP_ConnectionHandler *h, octet *buf, PortableServer_POA poa,
  	GIOP_MessageHeader *header, CORBA_Sequence_Octet *body,
	CORBA_Environment *env)
{
  GIOP_LocateReplyHeader *locate_reply_header;

  locate_reply_header = (GIOP_LocateReplyHeader *)newLocateReplyHeader();
  deMarshalLocateReply(locate_reply_header, body, buf, header);

  switch(locate_reply_header->locate_status){
     case GIOP_UNKNOWN_OBJECT:
	ORB_ERROR_OUT1("Unknown Object\n");
	break;
     case GIOP_OBJECT_HERE:
	 break;
     case GIOP_OBJECT_FORWARD:
     case GIOP_OBJECT_FORWARD_PERM:
	 ORB_ERROR_OUT1("Object forward\n");
         dumpMessage(buf + SIZEOF_GIOP_HEADER, header->message_size);
	 break;
     case GIOP_LOC_SYSTEM_EXCEPTION:
	 ORB_ERROR_OUT1("Locate System Exception\n");
         dumpMessage(buf + SIZEOF_GIOP_HEADER, header->message_size);
	 break;
     case GIOP_LOC_NEEDS_ADDRESSING_MODE:
	 ORB_ERROR_OUT1("Not Implemented\n");
         dumpMessage(buf + SIZEOF_GIOP_HEADER, header->message_size);
	 break;
     default:
	 ORB_ERROR_OUT2(" *[ %d ]*\n", (int)locate_reply_header->locate_status);
	 break;
  }
  RtORB_free(locate_reply_header,"GIOP_LocateReply_perform");
  return;
}

/*
 * free_request
 */
void free_request(void *arg){
  GIOP_Request_Item req = (GIOP_Request_Item)arg;
  if(!req) return;

  RtORB_free(req->buf, "free_request(buf)");
  RtORB_free(req->connh, "free_request(connh)");
  RtORB_free(req, "free_request");
  return;
}

/*
 * GIOP enqueu request
 */
PtrList *GIOP_enqueue_request(GIOP_ConnectionHandler *h, PtrList *lst){
//char buf[MaxMessageSize];
  char *buf = NULL;
  GIOP_MessageHeader header;
  GIOP_Request_Item request;

  buf = ( char* )malloc( MaxMessageSize );

  if(recieveMessage(h, &header, buf, MaxMessageSize) < 0){
    if ( buf ){
      free( buf );
    }
    return NULL;
  }

  request = (GIOP_Request_Item)RtORB_alloc(sizeof(GIOP_Request_struct),
		  "\nGIOP_enqueue_request");
  request->connh = RtORB_alloc(sizeof(GIOP_ConnectionHandler), "GIOP_enqueue_request");
  request->connh->type = h->type;
  request->connh->sock = h->sock;
#ifdef USE_SHMC
  request->connh->shmc = h->shmc;
#endif
    
  request->buf = (char *)RtORB_alloc(header.message_size + SIZEOF_GIOP_HEADER,
		  "GIOP_enqueue_request (buf)");

  memcpy(request->buf, buf, header.message_size + SIZEOF_GIOP_HEADER);

  lst = (PtrList *)PtrList_append(lst, request, free_request);

  if ( buf ){
    free( buf );
  }
  return lst;
}

/*
 *   GIOP Communication
 *
 */
int RecvMessage(GIOP_ConnectionHandler *h){
  int byte_order = 0 ;
  int fragment = -1 ;
//octet args[MaxMessageSize];
//char buf[MaxMessageSize];
  octet *args = NULL;
  char *buf = NULL;
  GIOP_MessageHeader header;

  CORBA_Sequence_Octet *body;
  CORBA_Environment env;

  args = ( octet* )malloc( MaxMessageSize * sizeof( octet ) );
  buf = ( char* )malloc( MaxMessageSize );

  memset(&header, 0, SIZEOF_GIOP_HEADER);
  memset(buf, 0, MaxMessageSize);
  memset(args, 0, MaxMessageSize);
  body = (CORBA_Sequence_Octet *)new_CORBA_Sequence_Octet(0);

  if(recieveMessage(h, &header, buf, MaxMessageSize) < 0){
    if ( args ) {
      free( args );
    }
    if ( buf ) {
      free( buf );
    }
    return -1;
  }

  if((header.flags & 0x01) == 0 ); /*header.message_size=ntohl(header.message_size); -> in recieveMessage */
  else byte_order = 1;
  if(header.version.minor > 0) fragment = header.flags & 0x02;

  ORB_ERROR_OUT2("[%d]\n", header.message_type);
  switch(header.message_type){
    case GIOP_Request: /// Sample Implementation only
         GIOP_Request_perform(h, buf, NULL, &header, body, &env);
         break;
    case GIOP_Reply:  // This is for a client, not implemented
         GIOP_Reply_perform(h, buf, NULL, &header, body, &env);
         break;
    case GIOP_CancelRequest: 
         GIOP_CancelRequest_perform(h, buf, NULL, &header, body, &env);
         break;
    case GIOP_LocateRequest:
         GIOP_LocationRequest_perform(h, buf, NULL, &header, body, &env);
         break;
    case GIOP_LocateReply:  // This is for a client, not complete
         GIOP_LocationReply_perform(h, buf, NULL, &header, body, &env);
         break;
    case GIOP_CloseConnection:
         GIOP_ConnectionHandler_close(h);
	 return -1;
    case GIOP_MessageError: 
	 ORB_ERROR_OUT1("Message Error occured\n");
         break;
    case GIOP_Fragment: // Not implemented
	 ORB_ERROR_OUT1("FragmentMessage have not implemented.....\n");
         break;
    default:
	 ORB_ERROR_OUT2("Invalid message type (%d)\n", header.message_type);

         if ( args ) {
           free( args );
         }
         if ( buf ) {
           free( buf );
         }
         return -1;
  }

  if ( args ) {
    free( args );
  }
  if ( buf ) {
    free( buf );
  }
  return 1;
}

void * GIOP_ConnectionHandler_get_arg(GIOP_ConnectionHandler *h);

/*
 *
 *
 *
 */
PtrList *GIOP_execute_request(PortableServer_POA poa, PtrList *lst){
  GIOP_ConnectionHandler *h;
  int version ;
  int byte_order = 0 ;
  int fragment = -1 ;
//octet args[MaxMessageSize];
  octet *args = NULL;
  char *buf;
  GIOP_Request_Item request;
  PtrList *current_request, *retval;
  GIOP_MessageHeader *header;
  CORBA_Sequence_Octet *body;
  CORBA_Environment env;

  args = ( octet* )malloc( sizeof( octet ) * MaxMessageSize );

  CORBA_MUTEX_LOCK();
//  memset(&buf, 0, MaxMessageSize);
  memset(args, 0, MaxMessageSize);

  body = (CORBA_Sequence_Octet *)new_CORBA_Sequence_Octet2(0);

  current_request = lst;

  CORBA_MUTEX_UNLOCK();

  while(current_request){

    CORBA_MUTEX_LOCK();

    request = (GIOP_Request_Item)current_request->item;
    header = (GIOP_MessageHeader *)request->buf;
    buf = request->buf;
    h = request->connh;

    poa = (PortableServer_POA) GIOP_ConnectionHandler_get_arg(h);

    version = header->version.minor;

    if((header->flags & 0x01) == 0 ) header->message_size=ntohl(header->message_size);
    else byte_order = 1;

    if(header->version.minor > 0) fragment = header->flags & 0x02;

    memset(&env, 0, sizeof(CORBA_Environment));

    CORBA_MUTEX_UNLOCK();


    switch(header->message_type){
      case GIOP_Request: /// Sample Implementation only
         CORBA_MUTEX_LOCK();
  
         GIOP_Request_perform(h, buf, poa, header, body, &env);

	 CORBA_MUTEX_UNLOCK();
         break;
      case GIOP_Reply:  // This is for a client, not implemented
	 CORBA_MUTEX_LOCK();
	 
         GIOP_Reply_perform(h, buf, poa, header, body, &env);

	 CORBA_MUTEX_UNLOCK();
         break;
      case GIOP_CancelRequest:
	 CORBA_MUTEX_LOCK();

         GIOP_CancelRequest_perform(h, buf, poa, header, body, &env);

	 CORBA_MUTEX_UNLOCK();
         break;
      case GIOP_LocateRequest:

	 CORBA_MUTEX_LOCK();

         GIOP_LocationRequest_perform(h, buf, poa, header, body, &env);

	 CORBA_MUTEX_UNLOCK();
         break;
      case GIOP_LocateReply:  // This is for a client, not complete

	 CORBA_MUTEX_LOCK();
	 
         GIOP_LocationRequest_perform(h, buf, poa, header, body, &env);

	 CORBA_MUTEX_UNLOCK();
         break;
      case GIOP_CloseConnection:
#ifndef USE_SHMC
	 close(h->sock);
         clear_socket_profile(h->sock, 0);
#endif
	 break;
      case GIOP_MessageError: 
	 ORB_ERROR_OUT1("Message Error occured\n");
         break;
      case GIOP_Fragment: // Not implemented
	 ORB_ERROR_OUT1("FragmentMessage have not implemented.....\n");
         break;
      default:
	 ORB_ERROR_OUT2("Invalid message type (%d)\n", header->message_type);
         break;
    }

     CORBA_MUTEX_LOCK();

     current_request->released = 1;  
     current_request = current_request->next;

     CORBA_MUTEX_UNLOCK();
  }


  CORBA_MUTEX_LOCK();
  
  delete_CORBA_Sequence_Octet2(body, 1);
  retval =  (PtrList *)PtrList_remove_released_items(lst);

  CORBA_MUTEX_UNLOCK();

  if ( args ){
    free ( args );
  }
  return retval;
}


int GIOP_ConnectionHandler_send(GIOP_ConnectionHandler *h, char *buf, int len)
{
  switch(h->type) {
  case CONNECTION_TYPE_SOCKET:
    return writeBytes(h->sock, buf, len);
#ifdef USE_SHMC   
  case CONNECTION_TYPE_SHMC:
    return shmcSendB(h->shmc, buf, len, NULL);
#endif    
  default:
    do_assert(0, "GIOP_ConnectionHandler_send : bad type");
    return -1;
  }
}

int GIOP_ConnectionHandler_recv(GIOP_ConnectionHandler *h, char *buf, int len)
{
  switch(h->type) {
  case CONNECTION_TYPE_SOCKET:
    return readBytes(h->sock, buf, len);
#ifdef USE_SHMC
  case CONNECTION_TYPE_SHMC:
    return shmcRecvB(h->shmc, buf, len, NULL);
#endif
  default:
    do_assert(0, "GIOP_ConnectionHandler_recv : bad type");
    return -1;
  }
}

void * GIOP_ConnectionHandler_get_arg(GIOP_ConnectionHandler *h)
{
  switch(h->type) {
  case CONNECTION_TYPE_SOCKET:
    return get_SockProfile_arg(h->sock);
#ifdef USE_SHMC
  case CONNECTION_TYPE_SHMC:
    return get_ShmcProfile_arg(h->shmc);
#endif
  default:
    do_assert(0, "GIOP_ConnectionHandler_get_arg");
    break;
  }
  return NULL;
}

void GIOP_ConnectionHandler_close(GIOP_ConnectionHandler *h)
{
  switch(h->type) {
  case CONNECTION_TYPE_SOCKET:
    close(h->sock);
    break;
#ifdef USE_SHMC
  case CONNECTION_TYPE_SHMC:
    shmcClose(h->shmc);
    break;
#endif
  default:
    break;
  }
}
