/*
 * Copyright (c) 2008, AIST.
 * All rights reserved. This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * National Institute of Advanced Industrial Science and Technology (AIST)
 */
/*
 * RtORB:
 *   Util.c
 *
 *  Copyright(C) 2006, Isao Hara, AIST
 *
 *  $Revision: 1.3 $
 *  $Date: 2008/05/21 05:11:30 $
 *  $Id: util.c,v 1.3 2008/05/21 05:11:30 yoshi Exp $
 */

#if USE_UUID
#include <uuid/uuid.h>
#else
#include <stdio.h>
//#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/time.h>
#endif

#include <RtORB/corba.h>
#include <RtORB/giop.h>
#include <RtORB/util.h>

#include <tinet/TINETWrapper.h>
#include <ethmac.h>
#include <stringlib.h>

#ifdef SEC
#undef SEC
#endif
#define SEC(systim) (systim / 1000)

#ifdef USEC
#undef USEC
#endif
#define USEC(systim) ((systim % 1000) * 1000)

/// toString
static char Char2Hex[]={
	 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0, 0, 0,
 	10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0 ,0, 0
};

/******* GIOP ************/
int readBytes(int fd, char *buf, int len){
  int res;
  char *buf_p = buf;
  while(len){
    res = recv(fd, buf_p, len, 0);
    if(res <= 0 ){
#if DEBUG
      ORB_ERROR_OUT2("Error in readByte (%d)\n", res);
#endif
      return -1; } 
    len -= res;
    buf_p += res;
  }
  return 1;
}

int writeBytes(int fd, char *buf, int len){
  int res;
  char *buf_p = buf;

  while(len){
    res = send(fd, buf_p, len, 0);
    if(res < 0 ){
#if DEBUG
	  ORB_ERROR_OUT2("Error in writeByte (%d)\n", res);
#endif
      return -1;
    }
    len -= res;
    buf_p += res;
  }
  return 1;
}
//////////////////////////////////////////////////////////////////////
///////////// Object ID ////

#if USE_UUID
char *new_ObjectID(){
  uuid_t uuid;
  char *id = (char *)RtORB_alloc(37, "new_ObjectID"); 

  uuid_generate(uuid);
  uuid_unparse(uuid, id);
  return id;
}
#else
char *new_ObjectID(){
  char id[36];
  SYSTIM systim;
  uint8_t mac_addr[6];

  get_tim( &systim );

  memset( id, 0, sizeof( id ) );
  memcpy( mac_addr, get_ifaddr(), 6 );

  t_vsnprintf_6( &id[0], sizeof( id ), "RtORB%02X%02X%02X%02X%02X%02X",
  				mac_addr[0],
  				mac_addr[1],
  				mac_addr[2],
  				mac_addr[3],
  				mac_addr[4],
  				mac_addr[5] );

  t_vsnprintf_1( &id[16], sizeof( id ) - 16, "%010X", SEC(systim) );
  t_vsnprintf_1( &id[26], sizeof( id ) - 26, "%06X", USEC(systim) );

  return (char *)RtORB_strdup(id, "new_ObjectID");
}
#endif

/////////////////////////////////////////////////////////////////////////////
// DEBUG
void  dumpMessage(unsigned char *buf, int size){
  int i;

  for(i=0;i<size;i++){
	ORB_MESSAGE_OUT2("%02x ", buf[i]);
	if((i % 8) == 7) ORB_MESSAGE_OUT1("\n");
  }
  ORB_MESSAGE_OUT1("\n");
  return;
}

void dump_value_by_typecode(void *val, CORBA_TypeCode tc){
#if DEBUG
  int i;
  SKIP_ALIAS(tc);

  switch(tc->kind){
    case tk_null:
    case tk_void:
	   ORB_ERROR_OUT1("NULL or void found\n");
       break;
    case tk_char:
    case tk_octet:
    case tk_boolean:
	   ORB_ERROR_OUT2("Octet = %c \n", *((char *)val));
       break;
    case tk_short:
    case tk_ushort:
	   ORB_ERROR_OUT2("Short = %d \n", *((short *)val));
       break;
    case tk_long:
    case tk_ulong:
    case tk_enum:
	   ORB_ERROR_OUT2("Long = %d \n", (int)(*((long *)val)));
       break;
    case tk_float:
	   ORB_ERROR_OUT2("Float = %f \n", *((float *)val));
       break;
    case tk_double:
	   ORB_ERROR_OUT2("Float = %lf \n", *((double *)val));
       break;
    case tk_string:
	   ORB_ERROR_OUT3("String = %s, 0x%x \n", *((char **)val),  (int)(*((char **)val)));
       break;
    case tk_sequence:
	   ORB_ERROR_OUT2("Sequence found --0x%x ---\n", val);
       break;
    case tk_except:
    case tk_struct:
	   ORB_ERROR_OUT2("Struct found-- 0x%x ----\n", val);
       for(i=0;i<tc->member_count;i++){
		 ORB_ERROR_OUT2("address val = %x\n", (int)val);
	 dump_value_by_typecode(val,  tc->member_type[i]);
	 val += size_of_typecode(tc->member_type[i]);
       }
       break;
    case tk_union:
	   ORB_ERROR_OUT1("Union found\n");
       break;
    case tk_objref:
	   ORB_ERROR_OUT1("Object found\n");
       break;
    case tk_any:
      {
	CORBA_any *any = (CORBA_any*)val;
	ORB_ERROR_OUT2("Any found : typecode = %d\n", any->_type->kind);
	void *v = CORBA_any_get_value(any);
	if (v) {
	  switch(any->_type->kind) {
	  case tk_null:
		ORB_ERROR_OUT1("Dump Any(null)\n");
	    break;
	  case tk_string:
		ORB_ERROR_OUT2("Dump Any(string) : %s\n", (char*)v);
	    break;
	    /*
	  case tk_objref:
		ORB_ERROR_OUT2("Dump Any(Object) : %s\n", (char*)any->_value+);
	    break;
	    */
	  default:
		ORB_ERROR_OUT1("Dump Any(not implemented)\n");
	    break;
	  }
	}
      }
      break;
    default:
	   ORB_ERROR_OUT2("Unknown typecode %d\n", (int)tc->kind);
       break;
  }
#endif

  return;
}

////////////////////////////////////////////////////////////
//
char ixdigit(char a){ return  Char2Hex[a -'0']; }

char hexdigit(char c){
   char buf = c & 0x0F;
   if (buf < 10) return '0' + buf;
   return 'a' + buf - 10;
}

octet *String2Octet(char *str){
  octet *result;
  int i,j,len2;
  int len = strlen(str);

  len2 = len >> 1;
  result = (octet *)RtORB_alloc(len2, "String2Octet");

  for(i=0;i<len2;i++){
    j=2*i;
    result[i] = (ixdigit(str[j]) << 4) | ixdigit(str[j+1]);
  }
  return result;
}

void
String__delete(char *str, char *info){
 if(str) RtORB_free(str, info);
}

long
RtORB_strlen(char *str){
 if (str) return strlen(str);
 return 0;
}

char *Get_IP_Address(){
  char *addr;
  int fd = socket(AF_INET,SOCK_STREAM, 0);
  addr = (char *)get_ip_address(fd);
  close(fd);

  return addr;
}
