/*!
 * @file TINETWrapper.c
 * @brief Wrapper functions of TINET
 * @date $Date: 2009-01-22 00:00:00 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: $
 *
 */


#include <kernel_cfg.h>
#include <RtORB/corba.h>
#include <RtORB/sockport.h>
#include <RtORB/ORBUtil.h>

#include <stringlib.h>
#include <tinet/TINETWrapper.h>

// REP(TCP t)(̂tinet_cfg.c)
extern T_TCP4_REP tcp_rep[];

// CEP(TCP ʐM[_)(̂tinet_cfg.c)
extern T_TCP4_CEP tcp_cep[];

// \PbgǗf[^
static S_TIW_DATA s_tiw_data[ ORB_TCP_CEP_NUM ];

// REP(TCP t)Ǘf[^(gp)
static int s_rep_info[ ORB_TCP_REP_NUM ];

// CEP(TCP ʐM[_)Ǘf[^(gp)
static int s_cep_info[ ORB_TCP_CEP_NUM ];

// m[hIPAhX
static UW s_my_ip = 0;

// IPԒʐMp[Nobt@
static char s_work_buf[ TIW_LOCAL_RECV_BUF ];

// SELECTǗf[^
static S_TIW_SELECT s_tiw_select[ TIW_MAX_SELECT_NUM ];

/*!
 * @if jp
 * @brief TINETbp[֐̏
 * @else
 * @endif
 */
int initTIW ( UW my_ip )
{
  T_IPV4EP  myaddr;
  T_IPV4EP  dstaddr;
  int     index;

  if ( my_ip > 0 )
  {
    s_my_ip = my_ip;
  }
  else
  {
#ifdef TIW_ETHMAC_NOT_USE
    // ethmac.cgp
    dstaddr.ipaddr  = 0;
    dstaddr.portno  = 0;
    myaddr.ipaddr = IPV4_ADDRANY;
    myaddr.portno = TCP_PORTANY;

    // AhXɐڑsAIP̐ݒsB
    tcp_con_cep( 1, &myaddr, &dstaddr, 0 );   // |[O

    for ( index = 0; index < ORB_TCP_CEP_NUM; index++ )
    {
      if (  tcp_cep[ index ].myaddr.ipaddr != 0 &&        // 0.0.0.0
          tcp_cep[ index] .myaddr.ipaddr != 0x7F000001 )    // 127.0.0.1
      {
        // IPAhXĂꍇ
        s_my_ip = tcp_cep[ index ].myaddr.ipaddr;
//        ORB_MESSAGE_OUT2( "IP Address = %08X\n", s_my_ip );
        break;
      }
    }
#else
    // ethmac.cgp
    s_my_ip = get_ipaddr();
//  ORB_MESSAGE_OUT2( "IP Address = %08X\n", s_my_ip );
#endif
  }

  memset( s_tiw_data, 0, sizeof( s_tiw_data ) );
  memset( s_rep_info, 0, sizeof( s_rep_info ) );
  memset( s_cep_info, 0, sizeof( s_cep_info ) );

  memset( s_tiw_select, 0, sizeof( s_tiw_select ) );

  if ( s_my_ip == 0 )
  {
    ORB_ERROR_OUT1( "IP Address Error\n" );
    return (-1);
  }
  return 0;
}

/*!
 * @if jp
 * @brief \Pbg̐(ʐM[_̊mۂs)
 * @else
 * @endif
 */
int socket ( int family, int type, int protocol )
{
  int lop1;
  int lop2;
  int status = (-1);

  ORB_SEM_LOCK( ORB_TIW_SEM1 );

  for ( lop1 = 0; lop1 < ORB_TCP_CEP_NUM && status < 0; lop1++ )
  {
    // 󂫃\Pbǧo
    if ( s_cep_info[ lop1 ] == 0 )
    {
      for ( lop2 = 0; lop2 < ORB_TCP_CEP_NUM; lop2++ )
      {
        if ( s_tiw_data[ lop2 ].status == TIW_SOCKET_IS_EMPTY )
        {
          // gpɏԕύX
          s_cep_info[ lop1 ] = 1;

          // \Pbg̏
          memset( &( s_tiw_data[ lop2 ] ), 0, sizeof( S_TIW_DATA ) );

          // \Pbg\ԂɕύX
          s_tiw_data[ lop2 ].status = TIW_SOCKET_IS_RESERVED;

          // CEP_IDݒ
          s_tiw_data[ lop2 ].cepid  = lop1 + ORB_TCP_CEPID_START;

          // \Pbghc͂P
          status = lop2 + 1;
          break;
        }
      }
    }
  }

  ORB_SEM_UNLOCK( ORB_TIW_SEM1 );

  if ( status < 0 ) 
  {
    // 󂫃\Pbg
    ORB_ERROR_OUT1( "No Socket\n" );
  }

  return status;
}

/*!
 * @if jp
 * @brief ڑ
 * @else
 * @endif
 */
int connect ( int s, const struct sockaddr *serv_addr, socklen_t addrlen )
{
  int status = (-1);
  T_IPV4EP  myaddr;
  struct sockaddr_in  sock_data_temp;
  struct sockaddr_in  *sock_temp_ptr;
  struct sockaddr_in  *sock_data;
  ER  ercd;
  int index;

  memset( &myaddr, 0, sizeof ( T_IPV4EP ) );

  if ( s == 0 || serv_addr == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(connect)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1].status != TIW_SOCKET_IS_RESERVED )
  {
    //ԃG[
    ORB_ERROR_OUT1( "Status Error(connect)\n" );
    return  status;
  }

  // GfBA^C~Őz
  sock_temp_ptr = ( struct sockaddr_in* )serv_addr;
  sock_data_temp.sin_addr.s_addr  = htonl( sock_temp_ptr->sin_addr.s_addr );
  sock_data_temp.sin_port     = htons( sock_temp_ptr->sin_port );

  sock_data = ( struct sockaddr_in* )&sock_data_temp;
  s_tiw_data[ s - 1].dstaddr.ipaddr = sock_data->sin_addr.s_addr;
  s_tiw_data[ s - 1].dstaddr.portno = sock_data->sin_port;

  myaddr.ipaddr = IPV4_ADDRANY;
  myaddr.portno = TCP_PORTANY;

  if ( s_tiw_data[ s - 1].dstaddr.ipaddr == s_my_ip )
  {
    // IPւ̐ڑ̏ꍇ
    for ( index = 0; index < ORB_TCP_CEP_NUM; index++ )
    {
      // Ώۃ\Pbǧ
      if (  ( s_tiw_data[ index ].status == TIW_SOCKET_IS_LISTENED    ) &&
          ( s_tiw_data[ index ].myaddr.portno == sock_data->sin_port  ) &&
          ( s_tiw_data[ index ].repid > 0 ) )

      {
        // ڑ҂ԁw|[g̃\Pbg݂ꍇ

        // \Pbg̎s̏(Listen)LZ
        tcp_can_cep( s_tiw_data[ index ].cepid, TFN_TCP_ALL );

        // \Pbg̐ڑݒ
        s_tiw_data[ index ].dstaddr.ipaddr = s_my_ip;
        s_tiw_data[ index ].dstaddr.portno = ( u_short )( s_tiw_data[ s - 1 ].cepid + TIW_PORT_BASE );
        s_tiw_data[ index ].target_socket_id = s;

        // \Pbg̃Xe[^XACCEPT҂ɕύX
        s_tiw_data[ index ].status = TIW_SOCKET_IS_WAIT_ACCEPT;

        // \Pbg̃\PbgIDf[^ɐݒ
        s_tiw_data[ s - 1 ].target_socket_id = index + 1;

        // Xe[^XڑɈڍs
        s_tiw_data[ s - 1 ].status = TIW_SOCKET_IS_CONNECTED;
        status = 0;

        // SelecttÕ`FbN
        setSelectFlag2( index + 1 );

        break;
      }
    }
  }
  else
  {
    // OIPAhX̏ꍇ
//    ercd = tcp_con_cep( s_tiw_data[ s - 1 ].cepid, &myaddr, &( s_tiw_data[ s - 1].dstaddr ), TMO_NBLK );  // mubLO[h
//    ercd = tcp_con_cep( s_tiw_data[ s - 1 ].cepid, &myaddr, &( s_tiw_data[ s - 1].dstaddr ), TMO_POL );   // |[O[h
    ercd = tcp_con_cep( s_tiw_data[ s - 1 ].cepid, &myaddr, &( s_tiw_data[ s - 1].dstaddr ), 15000 );    // ^CAEgL
    if ( ercd != E_OK && ercd != E_WBLK )
    {
      ORB_ERROR_OUT2( "tcp_con_cep Error = %d\n", ercd );
      return status;
    }

    // Xe[^XڑɈڍs
    s_tiw_data[ s - 1 ].status = TIW_SOCKET_IS_CONNECTED;
    status = 0;
  }

  return  status;
}

/*!
 * @if jp
 * @brief TCP t̐ݒ菈
 * @else
 * @endif
 */
int bind ( int s, const struct sockaddr *my_addr, socklen_t addrlen )
{
  struct sockaddr_in  sock_data_temp;
  struct sockaddr_in  *sock_temp_ptr;
  struct sockaddr_in  *sock_data;
  int lop;
  int status = (-1);

  if ( s == 0 || my_addr == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(bind)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status != TIW_SOCKET_IS_RESERVED )
  {
    //ԃG[
    ORB_ERROR_OUT1( "Status Error(bind)\n" );
    return  status;
  }

  // GfBA^C~Őz
  sock_temp_ptr = ( struct sockaddr_in* )my_addr;
  sock_data_temp.sin_addr.s_addr  = htonl( sock_temp_ptr->sin_addr.s_addr );
  sock_data_temp.sin_port     = htons( sock_temp_ptr->sin_port );

//  ORB_ERROR_OUT3( "Bind IP = %08X  Port = %d", sock_data_temp.sin_addr.s_addr, sock_data_temp.sin_port );

  sock_data = ( struct sockaddr_in* )&sock_data_temp;

  ORB_SEM_LOCK( ORB_TIW_SEM1 );

  for ( lop = 0; lop < ORB_TCP_REP_NUM; lop++ )
  {
    if ( tcp_rep[ lop ].myaddr.portno == sock_data->sin_port ||
    	tcp_rep[ lop ].myaddr.portno == TCP_PORTANY )
    {
      // ΉREP擾

      if ( s_rep_info[ lop ] == 0 )
      {
        s_tiw_data[ s - 1 ].status		= TIW_SOCKET_IS_BINDED;
        s_tiw_data[ s - 1 ].repid		= lop + ORB_TCP_REPID_START;
        if (tcp_rep[ lop ].myaddr.portno == TCP_PORTANY )
        {
          tcp_rep[ lop ].myaddr.portno = sock_data->sin_port;
        }
        s_tiw_data[ s - 1 ].myaddr.portno	= tcp_rep[ lop ].myaddr.portno;

        // oChς݂ɕύX
        s_rep_info[ lop ] = 1;

        status = 0;
      }
      else
      {
        // ɑ҂󂯏ԂɂȂĂꍇ
        ORB_ERROR_OUT1( "Status Error(bind)\n" );
      }
      break;
    }
  }

  ORB_SEM_UNLOCK( ORB_TIW_SEM1 );

  return status;
}

/*!
 * @if jp
 * @brief TCP t̐ڑ҂ԂɈڍs
 * @else
 * @endif
 */
int listen (int s, int backlog )
{
  int status = (-1);
  ER  ercd;

  if ( s == 0 )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(listen)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status != TIW_SOCKET_IS_BINDED )
  {
    // ԃG[
    ORB_ERROR_OUT1( "Status Error(listen)\n" );
    return  status;
  }

  // ڑv҂(󓮃I[v)
  ercd = tcp_acp_cep(	s_tiw_data[ s - 1 ].cepid,
			s_tiw_data[ s - 1 ].repid,
			&( s_tiw_data[ s - 1 ].dstaddr ),
			TMO_NBLK  );      // mubLO[h
  if ( ercd != E_OK && ercd != E_WBLK )
  {
    ORB_ERROR_OUT2( "tcp_acp_cep Error = %d\n", ercd );
    return status;
  }

  // Xe[^Xڑv҂Ɉڍs
  s_tiw_data[ s - 1 ].status = TIW_SOCKET_IS_LISTENED;
  status = 0;

  return status;
}

/*!
 * @if jp
 * @brief ڑ̏
 * @else
 * @endif
 */
int accept ( int s, struct sockaddr *dst_addr, socklen_t *addrlen )
{
  int status = (-1);
  int sock;
  S_TIW_DATA  temp_data;
  ER  ercd;

  if ( s == 0 || dst_addr == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(accept)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status != TIW_SOCKET_IS_WAIT_ACCEPT )
  {
    // ԃG[
    ORB_ERROR_OUT1( "Status Error(accept)\n" );
    return  status;
  }

  // f[^]p̃\Pbg擾
  sock = socket( 0, 0, 0 );
  if ( sock < 0 )
  {
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].dstaddr.ipaddr == s_my_ip )
  {
    // IP̐ڑ̏ꍇ
    if ( s_tiw_data[ s_tiw_data[ s - 1 ].target_socket_id - 1 ].target_socket_id == s )
    {
      // \Pbg̐ڑ\PbgԍXV
      s_tiw_data[ s_tiw_data[ s - 1 ].target_socket_id - 1 ].target_socket_id = sock;
    }
  }

  // repid̈p
  s_tiw_data[ sock - 1 ].repid = s_tiw_data[ s - 1 ].repid;

  // AhẌp
  s_tiw_data[ sock - 1 ].myaddr = s_tiw_data[ s - 1 ].myaddr;

  // VK\PbgƎۂɐڑĂ\Pbg
  temp_data = s_tiw_data[ sock - 1 ];
  s_tiw_data[ sock - 1 ] = s_tiw_data[ s - 1 ];
  s_tiw_data[ s - 1 ] = temp_data;

  // ڑς݃\Pbg̃Xe[^X̍XV
  s_tiw_data[ sock - 1 ].status = TIW_SOCKET_IS_CONNECTED;
  s_tiw_data[ sock - 1 ].repid  = 0;

  // ̒iKŐڑmB\Pbg̑҂󂯐̗LɊւ炸߂lɐݒ
  status = sock;

//  ORB_MESSAGE_OUT3( "New Accept cepid = %d repid = %d\n", s_tiw_data[ s - 1 ].cepid, s_tiw_data[ s - 1 ].repid );

  // VK\Pbgڑv҂(󓮃I[v)
  ercd = tcp_acp_cep(	s_tiw_data[ s - 1 ].cepid,
			s_tiw_data[ s - 1 ].repid,
			&( s_tiw_data[ s - 1 ].dstaddr ),
			TMO_NBLK  );      // mubLO[h
  if ( ercd != E_OK && ercd != E_WBLK )
  {
    ORB_ERROR_OUT2( "tcp_acp_cep Error = %d\n", ercd );
    return status;
  }

  // ڑ҂ԂɕύX
  s_tiw_data[ s - 1 ].status = TIW_SOCKET_IS_LISTENED;

  return status;
}

/*!
 * @if jp
 * @brief \PbgXe[^X̃`FbN
 * @else
 * @endif
 */
int select( int nfds, fd_set *readfds, fd_set *writefds, fd_set *errorfds, struct timeval *timeout )
{
  int index;
  int status = (-1);
  ER  ercd;
  char buf[4];
  long time_out;
  fd_set select_set;
  fd_set read_set;
#ifdef TIW_SELECT_CALLBACK_USE
  int select_index;
  FLGPTN wait_ptn;
  FLGPTN get_ptn;
  int recv_index;
#endif // TIW_SELECT_CALLBACK_USE

  if ( readfds == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(select)\n" );
    return  status;
  }

  // ^CAEgl̐ݒ(ms)
  time_out = ( timeout->tv_sec * 1000 ) + ( timeout->tv_usec / 1000 );

  // \Pbg0Ԃ͖gpׁ̈AɃNA
  FD_CLR( 0, readfds );

  // select̕ۑ
  select_set = *readfds;

  // ߂l̍XV
  status = 0;

  while (1)
  {
    // select̕
    *readfds = select_set;

    FD_ZERO( &read_set );

    for ( index = 0; index < ORB_TCP_CEP_NUM; index++ )
    {
      if ( FD_ISSET( index + 1, readfds ) )
      {
        // `FbNΏۂ̃\Pbg̏ꍇ
        if ( s_tiw_data[ index ].cepid == 0 ) 
        {
          // gp\Pbg̏ꍇ
          FD_CLR( index + 1, readfds );
        }
        else if( s_tiw_data[ index ].status == TIW_SOCKET_IS_WAIT_ACCEPT )
        {
          // ACCEPT҂Ԃ̏ꍇ
          status++;
        }
        else if( s_tiw_data[ index ].status == TIW_SOCKET_IS_CONNECTED )
        {
          // ڑԂ̏ꍇ

          if ( s_tiw_data[ index ].target_socket_id != 0 )
          {
            // IPʐM̏ꍇ

            if ( s_tiw_data[ index ].recv_size > 0 )
            {
              // Mf[^L
              status++;
            }
            else
            {
              // Mf[^
              FD_CLR( index + 1, readfds );
            }
          }
          else
          {
            // OIPƂ̒ʐM

            // O̎M҂\Pbg̐ݒ
            FD_SET( index + 1, &read_set );

            // ǂݍ݃TCY0ŎMs
            ercd = tcp_rcv_dat( s_tiw_data[ index ].cepid, buf, 0, TMO_POL );   // |[O
            if ( ercd < 0 )
            {
              // MG[
              FD_CLR( index + 1, readfds );
            }
            else
            {
              // Mf[^݂
              status++;
            }
          }
        }
        else
        {
          // ̑̃Xe[^X̏ꍇ
          FD_CLR( index + 1, readfds );
        }
      }
    }

    if ( status > 0 )
    {
      // Mf[^ or ڑvL̏ꍇ
      break;
    } 

#ifdef TIW_SELECT_CALLBACK_USE
    // R[obN[h

    if ( time_out > 0 )
    {
      ORB_CRITICAL_SECTION_START
      for ( select_index = 0; select_index < TIW_MAX_SELECT_NUM; select_index++ )
      {
        if ( s_tiw_select[ select_index ].enable == 0 )
        {
          // SelectǗ̋󂫂𔭌
          s_tiw_select[ select_index ].enable = 1;
          s_tiw_select[ select_index ].select_set = select_set;
          break;
        }
      }
      ORB_CRITICAL_SECTION_END

      if ( select_index < TIW_MAX_SELECT_NUM )
      {
         // SelectǗ̎擾ɐꍇ

        wait_ptn = ( 1 << select_index );

        // tȌ
        clr_flg( ORB_TIW_SELECT_FLG, ~(wait_ptn) );

        for ( recv_index = 0; recv_index < ORB_TCP_CEP_NUM; recv_index++ )
        {
          if ( FD_ISSET( recv_index + 1, &read_set ) )
          {
            // R[obNMp0oCgRead݂
            tcp_rcv_dat( s_tiw_data[ recv_index ].cepid, buf, 0, TMO_NBLK );   // mubLO
          }
        }

        // tO҂ԂɈڍs
        get_ptn = 0;
        ercd = twai_flg( ORB_TIW_SELECT_FLG, wait_ptn, TWF_ORW, &get_ptn, time_out );

        // ^CAEgl̃NA
        time_out = 0;

        ORB_CRITICAL_SECTION_START
        s_tiw_select[ select_index ].enable = 0;
        ORB_CRITICAL_SECTION_END

        if ( ercd == E_OK )
        {
           continue;
        }
      }
      else
      {
        ORB_ERROR_OUT1( "No Space(select)\n" );
      }
    }
#else
    // |[O[h
    if ( time_out >= TIW_POLING_DELAY )
    {
       // M҂Delay
       dly_tsk( TIW_POLING_DELAY );
       time_out -= TIW_POLING_DELAY;
       continue;
    }
    else if ( time_out > 0 )
    {
       // M҂Delay
       dly_tsk( time_out );
       time_out = 0;
       continue;
    }
#endif // TIW_SELECT_CALLBACK_USE

    if ( time_out <= 0 )
    {
       break;
    }
  }

//  ORB_ERROR_OUT2( "select status = %d", status );

  return status;
}

/*!
 * @if jp
 * @brief \PbgIvV̐ݒ(ɐI)
 * @else
 * @endif
 */
int setsockopt ( int s, int level, int optname, const void *optval, socklen_t optlen )
{
  // ɐI
  return 0;
}

/*!
 * @if jp
 * @brief \PbgIvV̎擾(ɐI)
 * @else
 * @endif
 */
int getsockopt ( int s, int level, int optname, void *optval, socklen_t *optlen )
{
  // ɐI
  return 0;
}

/*!
 * @if jp
 * @brief f[^M
 * @else
 * @endif
 */
int recv ( int s, void *buf, int len, int flags )
{
  int status = (-1);
  ER  ercd;
  int copy_size;

  if ( s == 0 || buf == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(recv)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status != TIW_SOCKET_IS_CONNECTED )
  {
    // ԃG[
    ORB_ERROR_OUT1( "Status Error(recv)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].target_socket_id != 0 )
  {
    // IPʐM̏ꍇ

    while( 1 )
    {
      if ( s_tiw_data[ s - 1 ].recv_size == 0 )
      {
        // M҂Delay
        dly_tsk( TIW_POLING_DELAY );
        continue;
      }

      ORB_SEM_LOCK( ORB_TIW_SEM2 );

      if ( s_tiw_data[ s - 1 ].recv_size > 0 )
      {
        // Mf[^L̏ꍇ

        if ( len >= s_tiw_data[ s - 1 ].recv_size )
        {
          // Mf[^TCYAobt@TCY傫ꍇ
          copy_size = s_tiw_data[ s - 1 ].recv_size;
        }
        else
        {
          // Mf[^TCYAobt@TCYꍇ
          copy_size = len;
        }

        // Mf[^̃Rs[
        memcpy( buf, s_tiw_data[ s - 1 ].recv_buf, copy_size );
        s_tiw_data[ s - 1 ].recv_size -= copy_size;

        if ( s_tiw_data[ s - 1 ].recv_size > 0 )
        {
          // Mf[^݂ꍇ͑Oɋl߂

          // [NɃRs[
          memcpy( s_work_buf,
                  &( s_tiw_data[ s - 1 ].recv_buf[ copy_size ] ),
                  s_tiw_data[ s - 1 ].recv_size );

          // obt@ɏ߂
          memcpy( s_tiw_data[ s - 1 ].recv_buf, s_work_buf, s_tiw_data[ s - 1 ].recv_size );
        }
        status = copy_size;
      }

      ORB_SEM_UNLOCK( ORB_TIW_SEM2 );
      break;
    }
  }
  else
  {
    // OIPƂ̒ʐM

    // M̎s
//  ercd = tcp_rcv_dat( s_tiw_data[ s - 1 ].cepid, buf, len, TMO_POL );   // |[O
    ercd = tcp_rcv_dat( s_tiw_data[ s - 1 ].cepid, buf, len, TMO_FEVR );  // ubLO
#ifdef TIW_SELECT_CALLBACK_USE
    if ( ercd == E_OBJ )
    {
      // mubLOɂĎMvς̏ꍇ̓LZɃgCs

      // s̏LZ
      tcp_can_cep( s_tiw_data[ s - 1 ].cepid, TFN_TCP_ALL );

      ercd = tcp_rcv_dat( s_tiw_data[ s - 1 ].cepid, buf, len, TMO_FEVR );  // ubLO
    }
#endif // TIW_SELECT_CALLBACK_USE
    if ( ercd < 0 )
    {
      ORB_ERROR_OUT2( "tcp_rcv_dat Error = %d\n", ercd );
      return status;
    }

    // Mf[^ݒ
    status = ercd;
  }
  return status;
}

/*!
 * @if jp
 * @brief f[^M
 * @else
 * @endif
 */
int send ( int s, const void *buf, int len, int flags )
{
  int status = (-1);
  ER  ercd;
  int remain_size = len;
  int send_size = 0;
  int copy_size;

  if ( s == 0 || buf == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(send)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status != TIW_SOCKET_IS_CONNECTED )
  {
    // ԃG[
    ORB_ERROR_OUT1( "Status Error(send)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].target_socket_id != 0 )
  {
    int target_socket = s_tiw_data[ s - 1 ].target_socket_id;

    do
    {
      // IPʐM̏ꍇ
      ORB_SEM_LOCK( ORB_TIW_SEM2 );

      if ( ( TIW_LOCAL_RECV_BUF - s_tiw_data[ target_socket - 1 ].recv_size ) >= remain_size )
      {
        // M̋󂫃obt@MTCYȏ̏ꍇ
        copy_size = remain_size;
      }
      else
      {
        // M̋󂫃obt@MTCY̏ꍇ
        copy_size =  TIW_LOCAL_RECV_BUF - s_tiw_data[ target_socket - 1 ].recv_size;
      }

      // Mobt@Ƀf[^Rs[
      memcpy( &( s_tiw_data[ target_socket - 1 ].recv_buf[ s_tiw_data[ target_socket - 1 ].recv_size ] ), &buf[ send_size ], copy_size );
      s_tiw_data[ target_socket - 1 ].recv_size += copy_size;

      ORB_SEM_UNLOCK( ORB_TIW_SEM2 );

      // MσTCYZ
      send_size += copy_size;

      // Mf[^TCY̌Z
      remain_size -= copy_size;

      // SelecttÕ`FbN
      setSelectFlag2( target_socket );

      if ( remain_size > 0 )
      {
        // M҂Delay
        dly_tsk( TIW_POLING_DELAY );
      }
    } while ( remain_size > 0 );

    // Mf[^ݒ
    status = send_size;
  }
  else
  {
    // OIPƂ̒ʐM

    do
    {
      // M̎s
//    ercd = tcp_snd_dat( s_tiw_data[ s - 1 ].cepid, &buf[ send_size ], remain_size, TMO_POL );   // |[O
      ercd = tcp_snd_dat( s_tiw_data[ s - 1 ].cepid, &buf[ send_size ], remain_size, TMO_FEVR );  // ubLO
      if ( ercd == E_OBJ )
      {
        // yfBÕR}h݂ꍇ͏܂őҋ@
        ercd = 0;
      }
      else if ( ercd < 0 )
      {
        ORB_ERROR_OUT2( "tcp_snd_dat Error = %d\n", ercd );
        return status;
      }

      // MσTCYZ
      send_size += ercd;

      // Mf[^TCY̌Z
      remain_size -= ercd;

      if ( remain_size > 0 )
      {
        // M҂Delay
        dly_tsk( TIW_POLING_DELAY );
      }
    } while ( remain_size > 0 );

    // Mf[^ݒ
    status = send_size;
  }

  return status;
}

/*!
 * @if jp
 * @brief SOCKET̃N[Y
 * @else
 * @endif
 */
int close( int s )
{
  int status = (-1);

  if ( s == 0 )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(close)\n" );
    return  status;
  }

  ORB_SEM_LOCK( ORB_TIW_SEM1 );

  switch ( s_tiw_data[ s - 1 ].status ) {

  case TIW_SOCKET_IS_RESERVED:    // \Pbĝ݊m
  case TIW_SOCKET_IS_BINDED:      // BIND
    // sv
    break;

  case TIW_SOCKET_IS_WAIT_CONNECT:  // ڑ҂
  case TIW_SOCKET_IS_LISTENED:    // LISTEN

    // s̏LZ
    tcp_can_cep( s_tiw_data[ s - 1 ].cepid, TFN_TCP_ALL );

    // ʐM[_̃N[Y(breakȂ)

  case TIW_SOCKET_IS_CONNECTED:   // ڑ
  case TIW_SOCKET_IS_WAIT_ACCEPT:   // ACCEPT҂

    if ( s_tiw_data[ s - 1 ].target_socket_id == 0 )
    {
      // IPʐMȊȌꍇ

      // ʐM[_̃N[Y
      tcp_cls_cep( s_tiw_data[ s - 1 ].cepid, TMO_NBLK ); // mubLO[h
    }

    break;
  }

  if ( s_tiw_data[ s - 1 ].repid > 0 )
  {
    // (TCP t)Ǘf[^𖢎gpɕύX
    s_rep_info[ s_tiw_data[ s - 1 ].repid - ORB_TCP_REPID_START] = 0;
  }

  // (TCP ʐM[_)Ǘf[^𖢎gpɕύX
  s_cep_info[ s_tiw_data[ s - 1 ].cepid - ORB_TCP_CEPID_START] = 0;

  // \PbgǗf[^̃NA
  memset( &( s_tiw_data[ s - 1 ] ), 0, sizeof( S_TIW_DATA ) );

  ORB_SEM_UNLOCK( ORB_TIW_SEM1 );

//  ORB_ERROR_OUT1( "close End\n" );

  return status;
}

/*!
 * @if jp
 * @brief TINET̃R[obN֐
 * @else
 * @endif
 */

ER
callback_nblk_tiw (ID cepid, FN fncd, VP p_parblk)
{
  int lop;
  ER  error = E_OK;

  switch ( fncd )
  {

  case TFN_TCP_ACP_CEP:       // ڑv
    if ((ER)p_parblk == E_OK)
    {
//    ORB_MESSAGE_OUT2( "TFN_TCP_ACP_CEP cepid = %d\n", cepid );
      for ( lop = 0; lop < ORB_TCP_CEP_NUM; lop++ )
      {
        if ( s_tiw_data[ lop ].cepid == cepid )
        {
          // ΉCEP擾
//        ORB_MESSAGE_OUT2( "TFN_TCP_ACP_CEP socket = %d\n", lop + 1 );
          // \Pbgaccept҂ԂɕύX
          s_tiw_data[ lop ].status = TIW_SOCKET_IS_WAIT_ACCEPT;

          // SelecttÕ`FbN
          setSelectFlag2( cepid );

          break;
        }
      }

    }
    break;

  case TFN_TCP_RCV_DAT:       // Mv
//  ORB_MESSAGE_OUT3( "TFN_TCP_RCV_DAT CEPID = %d p_parblk = %d", cepid, (ER)p_parblk );
    if ((ER)p_parblk >= 0)
    {
      // SelecttÕ`FbN
      setSelectFlag2( cepid );
    }
    break;

  case TFN_TCP_CLS_CEP:
  case TFN_TCP_RCV_BUF:
  case TFN_TCP_GET_BUF:
  case TFN_TCP_SND_DAT:       // Mv
  case TFN_TCP_CON_CEP:
  case TFN_TCP_SND_OOB:
  default:
//  ORB_ERROR_OUT3( "Unknown CallBack cepid = %d fnc = %d\n", cepid, fncd );
    error = E_PAR;
    break;
  }
  return error;
}

/*!
 * @if jp
 * @brief \Pbg̎擾
 * @else
 * @endif
 */
int getsockname ( int s, struct sockaddr *name, socklen_t *name_len )
{
  int status = (-1);
  struct sockaddr_in  *sock_data;

  if ( s == 0 || name == NULL )
  {
    // G[
    ORB_ERROR_OUT1( "Argument Error(getsockname)\n" );
    return  status;
  }

  if ( s_tiw_data[ s - 1 ].status == TIW_SOCKET_IS_EMPTY )    // 󂫏Ԃ̓G[
  {
    // ԃG[
    ORB_ERROR_OUT1( "Status Error(getsockname)\n" );
    return  status;
  }

  // \Pbg̐ݒ
  sock_data = ( struct sockaddr_in * )name;
  sock_data->sin_addr.s_addr = htonl( s_my_ip );

  if ( s_tiw_data[ s - 1 ].repid > 0 )
  {
    // ڑ҂|[g̏ꍇ
    sock_data->sin_port = htons( s_tiw_data[ s - 1 ].myaddr.portno );
  }
  else
  {
    // ڑς݃|[g̏ꍇ

    // TINETłCEPɊꂽPortԍ擾łȂ
    // CEPx[Xɍ쐬(|[gԍł͂ȂƂɒ)B
    sock_data->sin_port = ( u_short )( s_tiw_data[ s - 1 ].cepid + TIW_PORT_BASE );
  }

  status = 0;
  return status;
}

/*!
 * @if jp
 * @brief IPAhX̎擾
 * @else
 * @endif
 */
struct hostent * gethostbyname( char *name )
{
  return NULL;
}

/*!
 * @if jp
 * @brief Select҂\Pbg̃`FbN(Socketԍ)
 * @else
 * @endif
 */
static void setSelectFlag1( int s )
{
#ifdef TIW_SELECT_CALLBACK_USE
  FLGPTN flg = 0;
  int index;

  for ( index = 0; index < TIW_MAX_SELECT_NUM; index++ )
  {
    if ( s_tiw_select[index].enable == 1 )
    {
      if( FD_ISSET( s, &(s_tiw_select[index].select_set ) ) )
      {
        // `FbNΏۃ\Pbg̏ꍇ
        flg += ( 1 << index );
      }
    }
  }

  if ( flg > 0 )
  {
    set_flg( ORB_TIW_SELECT_FLG, flg );
  }
#endif // TIW_SELECT_CALLBACK_USE
}

/*!
 * @if jp
 * @brief Select҂\Pbg̃`FbN(CEP ID)
 * @else
 * @endif
 */
static void setSelectFlag2( ID cepid )
{
#ifdef TIW_SELECT_CALLBACK_USE
  int index;

  for ( index = 0; index < ORB_TCP_CEP_NUM; index++ )
  {
    if ( s_tiw_data[ index ].cepid == cepid )
    {
      // CEP ID\Pbgԍ擾
      setSelectFlag1( index + 1 );
      break;
    }
  }
#endif // TIW_SELECT_CALLBACK_USE
}
