/*!
 * @file TINETWrapper.h
 * @brief Wrapper functions of TINET
 * @date $Date: 2008-12-15 00:00:00 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: $
 *
 */


#ifndef __TINETWRAPPER_H__
#define __TINETWRAPPER_H__

#ifdef __cplusplus
extern "C" {
#endif  // __cplusplus

#include "tinet_id.h"
#include <netinet/in.h>
#include <netinet/in_itron.h>
#include <tinet_defs.h>
#include <tinet_config.h>
#include <netinet/in_var.h>
#include <net/net.h>

#include <net/if.h>
#include <netinet/ip.h>
#include <net/ethernet.h>
#include <netinet/tcp.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>

#include <tinet/compatibility.h>
#include <TINETWrapperDef.h>

#define TIW_SELECT_CALLBACK_USE                 // Select҂Callbackgp

#define TIW_LOCAL_RECV_BUF	TCP_RWBUF_SIZE	// IPԒʐMpMobt@TCY

#define TIW_PORT_BASE		10000		// |[gԍx[X(CEPIDZl|[gԍ)

#define TIW_POLING_DELAY	10		// select/recṽ|[OԊu

#define	TIW_MAX_SELECT_NUM	4		// Ɏs\select


//////////////////////////////////////////////////////////////
//    \PbgǗf[^
//////////////////////////////////////////////////////////////

typedef struct {
  ID		cepid;			// CEP ID
  ID		repid;			// REP ID(҂󂯃\Pbĝݐݒ)

  T_IPV4EP	myaddr;			// m[hAhX
  T_IPV4EP	dstaddr;		// ڑΏۃAhX

  int		status;			// \Pbg̏
#define TIW_SOCKET_IS_EMPTY		0	// gp
#define TIW_SOCKET_IS_RESERVED		1	// \Pbĝ݊m
#define TIW_SOCKET_IS_WAIT_CONNECT	2	// ڑ҂
#define TIW_SOCKET_IS_BINDED		3	// BIND
#define TIW_SOCKET_IS_LISTENED		4	// LISTEN
#define TIW_SOCKET_IS_WAIT_ACCEPT	5	// ACCEPT҂
#define TIW_SOCKET_IS_CONNECTED		6	// ڑ

  int   target_socket_id;		// IPԒʐM̑\Pbgԍ
  int   recv_size;			// IPԒʐMpMobt@gpTCY
  char  recv_buf[TIW_LOCAL_RECV_BUF];	// IPԒʐMpMobt@
} S_TIW_DATA;

//////////////////////////////////////////////////////////////
//    SELECTǗf[^
//////////////////////////////////////////////////////////////

typedef struct {
  int    enable;     // 0:gpA1:gp
  fd_set select_set; // select̃\Pbg
} S_TIW_SELECT;

//////////////////////////////////////////////////////////////
//    TINETbp[֐
//////////////////////////////////////////////////////////////

int initTIW ( UW );
int socket ( int, int, int );
int connect ( int, const struct sockaddr *, socklen_t );
int bind ( int, const struct sockaddr *, socklen_t );
int listen ( int, int );
int accept ( int, struct sockaddr *, socklen_t * );
int select( int, fd_set *, fd_set *, fd_set *, struct timeval * );
int getsockopt ( int ,int ,int ,void *, socklen_t * );
int setsockopt ( int ,int ,int ,const void *, socklen_t );
int recv ( int, void *, int , int );
int send ( int, const void *, int , int );
int close( int );
int getsockname ( int, struct sockaddr *, socklen_t * );
struct hostent * gethostbyname( char * );

static void setSelectFlag1( int );
static void setSelectFlag2( ID );

#ifdef __cplusplus
}
#endif  // __cplusplus

#endif  // __TINETWRAPPER_H__
