// -*- C++ -*-
/*!
 * @file OS.k
 * @brief OS class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_OS_H
#define COIL_OS_H

#include <kernel.h>
#include <string.h>
#include <coil/String.h>
#include "target_syssvc.h"
#include "ethmac.h"
#include "stringlib.h"

extern "C"
{
  extern char *optarg;
};

namespace coil
{
#define SYS_NAME "TOPPERS/ASP Kernel"
#define UN_NAME_SIZE  64
#define UN_VERSION_SIZE  32
  typedef struct utsname_ {
    char sysname[UN_NAME_SIZE];
    char nodename[UN_NAME_SIZE];
    char release[UN_VERSION_SIZE];
    char version[UN_VERSION_SIZE];
    char machine[UN_NAME_SIZE];
  }utsname;

  inline int uname(utsname* name)
  {
    uint8_t major_version;
    uint8_t minor_version;
    uint8_t patch_version;
    uint8_t* macaddr;
    int i;

    if (name) {
      // OS̾
      strcpy(name->sysname, SYS_NAME);

      // Ρ̾MACɥ쥹@@֡
      macaddr = get_ifaddr();
      for (i = 0; i < 5; i++) {
        t_vsnprintf(&name->nodename[i], 3, "%2.2X:", macaddr[i]);
      }
      t_vsnprintf(&name->nodename[i], 3, "%2.2X", macaddr[i]);

      // ͥΥСֹ
      major_version = (uint8_t)(((TKERNEL_PRVER) >> 12) & 0x000f);
      minor_version = (uint8_t)(((TKERNEL_PRVER) >> 4) & 0x00ff);
      patch_version = (uint8_t)(TKERNEL_PRVER & 0x000f);
      t_vsnprintf(name->release, UN_VERSION_SIZE, "%d.%d.%d",
              major_version, minor_version, patch_version);

      // ͥͤΥСֹ
      major_version = (uint8_t)(((TKERNEL_SPVER) >> 4) & 0x000f);
      minor_version = (uint8_t)(TKERNEL_SPVER & 0x000f);
      t_vsnprintf(name->version, UN_VERSION_SIZE, "1.%X.%X",
              major_version, minor_version);

      // ϡɥμ
      strcpy(name->machine, TARGET_NAME);

      return 0;
    }
    return -1;
  }

  typedef ::pid_t pid_t;
  inline pid_t getpid()
  {
    ID tskid;
    get_tid(&tskid);
    return (pid_t)tskid;
  }
};

#endif // COIL_OS_H
