// -*- C++ -*-
/*!
 * @file Task.cpp
 * @brief Task class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_TASK_H
#define COIL_TASK_H

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>

#define	SVC_PERROR(expr) coil::svc_perror(__FILE__, __LINE__, #expr, (expr))

namespace coil
{
  Inline void svc_perror(const char *file, int_t line,
		  const char *expr, ER ercd);

  class Task
  {
  public:
    Task();
    Task(ID tskid);
    virtual ~Task();
    virtual int open(void* args = 0);
    virtual int close(unsigned long flags = 0);
    virtual int svc();
    virtual void activate();
    virtual int wait(void);
    virtual int suspend(void);
    virtual int resume(void);
    static void* svc_run(void* args = 0);
    static void dly_until(unsigned long* plasttim, unsigned long delay);

  private:
    int m_count;
    ID m_tskid;
    void* m_args;
  };

  /*!
   * ӥΥ顼Υ
   */
  Inline void
  svc_perror(const char *file, int_t line, const char *expr, ER ercd)
  {
  	if (ercd < 0) {
  		t_perror(LOG_ERROR, file, line, expr, ercd);
  	}
  }
};

#endif // COIL_TASK_H
