// -*- C++ -*-
/*!
 * @file Time.h
 * @brief Time functions
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_TIME_H
#define COIL_TIME_H

#include <coil/config_coil.h>
#include <coil/TimeValue.h>
#include <kernel.h>
#include <t_syslog.h>

namespace coil
{
#define SEC_TO_RELTIM(sec) ((sec) * 1000)
#define USEC_TO_RELTIM(usec) ((usec) / 1000)
#define TVAL_TO_RELTIM(tval) (SEC_TO_RELTIM((tval).sec()) + USEC_TO_RELTIM((tval).usec()))
#define TVAL_TO_SYSTIM(tval) TVAL_TO_RELTIM(tval)
#define SEC(systim) ((systim) / 1000)
#define USEC(systim) (((systim) % 1000) * 1000)

  typedef long time_t;
  typedef long suseconds_t;
  typedef long useconds_t;

  typedef struct timeval {
    time_t      tv_sec;     /*  */
    suseconds_t tv_usec;    /* ޥ */
  };

  typedef struct timezone {
    int tz_minuteswest;     /* ˥åɸȤκ (ʬñ) */
    int tz_dsttime;         /* ƻĴη */
  };

  inline ER sleep(unsigned int seconds)
  {
    RELTIM dlytim = SEC_TO_RELTIM(seconds);
    return dly_tsk(dlytim);
  }

  inline int sleep(TimeValue& interval)
  {
    RELTIM dlytim = TVAL_TO_RELTIM(interval);
    return dly_tsk(dlytim);
  }

  inline int usleep(useconds_t usec)
  {
    RELTIM dlytim = USEC_TO_RELTIM(usec);
    return dly_tsk(dlytim);
  }

  inline ER gettimeofday(struct timeval *tv, struct timezone *tz)
  {
    SYSTIM systim;
    ER ercd = get_tim(&systim);
    if (ercd >= 0) {
      tv->tv_sec = SEC(systim);
      tv->tv_usec = USEC(systim);
    }
    return ercd;
  }

  inline TimeValue gettimeofday()
  {
    timeval tv;
    gettimeofday(&tv, 0);
    return TimeValue(tv.tv_sec, tv.tv_usec);
  }

  inline int settimeofday(const struct timeval *tv , const struct timezone *tz)
  {
    return 0;  // no implementation
  }

  inline void sleep_until(TimeValue* lasttime, TimeValue& interval)
  {
    RELTIM dlytim = 0;			/* ٱ䤹 */
    SYSTIM curtim;			/* ߥƥ */
    RELTIM deley = TVAL_TO_RELTIM(interval);
    SYSTIM lasttim = TVAL_TO_SYSTIM(*lasttime);
    SYSTIM nexttim = lasttim + deley;
    					/* ư */

    if (get_tim(&curtim) != E_OK) {
      return;
    }

    if (lasttim > curtim) {
      /* ߥƥ֤Сե */
      if (nexttim < lasttim) {
	if (nexttim > curtim) {
	  /* ư֤Сե */
	  dlytim = nexttim - curtim;
	}
      }
    } else if (nexttim < lasttim) {
      /* ߥƥ֤СեǤϤʤư֤Сե */
      dlytim = nexttim + (ULONG_MAX - curtim);
    } else {
      /* ߥƥ֤ȼư֤Ȥ˥СեǤʤ */
      if (nexttim > curtim) {
	dlytim = nexttim - curtim;
      }
    }

    dly_tsk(dlytim);

    if (dlytim != 0) {
      lasttim += deley;
    } else {
      lasttim = curtim;
    }
    *lasttime = TimeValue(SEC(lasttim), USEC(lasttim));
  }

};

#endif // COIL_TIME_H
