// -*- C++ -*-
/*!
 * @file TopppersTask.cpp
 * @brief Task class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#include <coil/Task.h>

namespace coil
{
  Task::Task()
    : m_count(0)
  {
  }

  Task::Task(ID tskid)
    : m_count(0), m_tskid(tskid)
  {
  }

  Task::~Task()
  {
    if ( m_count > 0 )
    {
      ext_tsk();
    }
    m_count = 0;
  }

  int Task::open(void* args)
  {
    return 0;
  }

  int Task::close(unsigned long flags)
  {
    return 0;
  }

  int Task::svc()
  {
    return 0;
  }

  void Task::activate()
  {
    if (m_count == 0) {
      SVC_PERROR(act_tsk(m_tskid));
      m_count++;
    }
  }

  int Task::wait(void)
  {
    return 0;
  }

  int Task::suspend(void)
  {
    return 0;
  }

  int Task::resume(void)
  {
    return 0;
  }

  void* Task::svc_run(void* args)
  {
    Task* t = (coil::Task*)args;
    int status;
    status = t->svc();
    return 0;
  }

  /*
   *  plassttim : ŌɃX[vԁiPʁF~bj
   *  delay     : x鎞ԁiPʁF~bj
   */
  void Task::dly_until(unsigned long* plasttim, unsigned long delay)
  {
    RELTIM dlytim = 0;			/* x鎞 */
    SYSTIM curtim;			/* ݃VXe */
    SYSTIM nexttim = *plasttim + delay;	/* N */

    if (get_tim(&curtim) != E_OK) {
      return;
    }

    /*
     *  ݃VXe(curtim)ŌɃX[v(lasttim)
     *  ɐił邱Ƃ
     *
     *  @ʏAI[o[t[Ȃȉ̐}̂悤ɂȂ
     *          lasttim        curtim     nexttim
     *                                   
     *  (0)|---------------------------------------------------------|(ULONG_MAX)
     *    ̏ꍇAX[v鎞Ԃ́A(nexttim - curtim) ԁiPʁF~bj
     *
     *  A݃VXeԂI[o[t[ꍇ
     *      curtim         nexttim                       lasttim
     *                                                 
     *  (0)|---------------------------------------------------------|(ULONG_MAX)
     *    NԂI[ot[ĂȂAݎԂO̎
     *    ƂƂȂ̂ŃX[vKvȂB
     *    NԂI[o[t[ݎԂ̎Ԃłꍇ
     *    X[vBi̐}̂悤Ɂj
     *    X[v鎞Ԃ́A(nexttim - curtim) ԁiPʁF~bj
     *
     *  BNԂI[o[t[ꍇ
     *          nexttim       lasttim                 curtim
     *                                             
     *  (0)|---------------------------------------------------------|(ULONG_MAX)
     *    ݎԂI[o[t[ȂAƓlB
     *    ŌɃX[vԂI[o[t[łȂȂAݎԂ
     *    l܂łƉl玟NԂ܂łX[v鎞
     *    ((ULOG_MAX - curtim) + nexttim ) ԁiPʁF~bj
     */

    if (*plasttim > curtim) {
      /* ݃VXeԂI[o[t[ */
      if (nexttim < *plasttim) {
	if (nexttim > curtim) {
	  /* NԂI[o[t[ */
	  dlytim = nexttim - curtim;
	}
      }
    } else if (nexttim < *plasttim) {
      /* ݃VXeԂI[o[t[ł͂ȂANԂI[o[t[ */
      dlytim = nexttim + (ULONG_MAX - curtim);
    } else {
      /* ݃VXeԂƎNԂƂɃI[o[t[łȂ */
      if (nexttim > curtim) {
	dlytim = nexttim - curtim;
      }
    }

    dly_tsk(dlytim);

    if (dlytim != 0) {
      *plasttim += delay;
    } else {
      *plasttim = curtim;
    }
  }
};
