/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2008 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2006-2008 by Task-intelligence Research Group,
 *             Intelligent Systems Research Institute,
 *             National Institute of
 *               Advanced Industrial Science and Technology (AIST), Japan
 * 
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include <dbgu_serial.h>
#include "OpenRTMSetup.h"
#include <rtm/DefaultConfiguration.h>

/*-----------------------------------------------------------
	Definitions
-----------------------------------------------------------*/
#define	ORTMS_MAX_LEN 64

/* character constants */
#define LF      '\n'
#define CR      '\r'
#define BS      0x08
#define DEL     0x7F
#define ISWSP(c) ((c)==' ' || (c)=='\t' || (c)=='\r' || (c)=='\n')

#define DLM_CFG	':'

struct ParamInfo {
  uint8_t* name;				/* p[^ */
  bool_t (*param_valid)(uint8_t*, uint8_t);	/* ͔֐ */
};

/*-----------------------------------------------------------
	Private Variables
-----------------------------------------------------------*/
static const uint8_t s_magic_word[]  = "OpenRTM";
static const uint8_t s_name_server_param_name[] = "corba.nameservers";

static const uint8_t *s_bool_name[] = {
	(uint8_t *)"YES", (uint8_t *)"yes",
	(uint8_t *)"NO", (uint8_t *)"no",
	NULL
};
static const uint8_t *s_level_name[] = {
	(uint8_t *)"SILENT",
	(uint8_t *)"ERROR",
	(uint8_t *)"WARN",
	(uint8_t *)"INFO",
	(uint8_t *)"NORMAL",
	(uint8_t *)"DEBUG",
	(uint8_t *)"TRACE",
	(uint8_t *)"VERBOSE",
	(uint8_t *)"PARANOID",
	(uint8_t *)"MANDATORY",
	(uint8_t *)"silent",
	(uint8_t *)"error",
	(uint8_t *)"warn",
	(uint8_t *)"info",
	(uint8_t *)"normal",
	(uint8_t *)"debug",
	(uint8_t *)"trace",
	(uint8_t *)"verbose",
	(uint8_t *)"paranoid",
	(uint8_t *)"mandatory",
	NULL
};

/*-----------------------------------------------------------
	Public Variables
-----------------------------------------------------------*/
extern void *end;

/*-----------------------------------------------------------
	Private Functions
-----------------------------------------------------------*/
static void s_putc(uint8_t c)
{
  if (c == '\n') {
    ep93xx_putc('\r');
  }
  ep93xx_putc(c);
}
static void s_puts(uint8_t* s)
{
  uint8_t* t;

  for (t = s; *t != '\0'; t++) {
    if (*t == '\n') {
      ep93xx_putc('\r');
    }
    ep93xx_putc(*t);
  }
}

static int_t s_strcmp(uint8_t* s1, uint8_t* s2)
{
  while ( *s1 != '\0' && *s2 != '\0' ) {
    if ( *s1 != *s2 ) {
      return 1;
    }
    s1++;
    s2++;
  }
  if ( *s1 != '\0' || *s2 != '\0' ) {
    return 1;
  }
  return 0;
}

static int_t s_strncmp(uint8_t* s1, uint8_t* s2, int_t len)
{
  int_t lop;

  for ( lop = 0; lop < len; lop++, s1++, s2++ ) {
    if ( *s1 == '\0' || *s2 == '\0' ) {
      return 1;
    }
    if ( *s1 != *s2 ) {
      return 1;
    }
  }
  return 0;
}

static int_t s_strlen(uint8_t* s)
{
  int_t len = 0;
  while ( *s != '\0' ) {
    len++;
    s++;
  }

  return len;
}

static int_t s_strcpy(uint8_t* d, uint8_t* s)
{
  uint8_t *st = d;

  for ( ; *s != '\0'; d++, s++) {
    *d = *s;
  }
  *d = '\0';

  return (int_t)(d - st);
}
static int_t s_strncpy(uint8_t* d, uint8_t* s, int_t l)
{
  int_t i;
  uint8_t *st = d;

  for (i = 0; i < l - 1 && *s != '\0'; i++, d++, s++) {
    *d = *s;
  }
  *d = '\0';

  return (int_t)(d - st);
}

static int_t s_copy_default( uint8_t const * param_name, uint8_t* value )
{
  uint8_t **check = (uint8_t**)default_config;
  while ( *check[0] !=  '\0' ) {
    if ( s_strcmp( (uint8_t *)param_name, *check ) == 0 ) {
      check++;
      s_strcpy( value, *check );
      return 0;
    }
    else {
      check++;
    }
    check++;
  }

  return 1;
}

static int_t s_ncopy_default( uint8_t const * param_name, uint8_t* value, uint8_t len )
{
  uint8_t **check = (uint8_t**)default_config;
  while ( *check[0] !=  '\0' ) {
    if ( s_strcmp( (uint8_t *)param_name, *check ) == 0 ) {
      check++;
      s_strncpy( value, *check, len );
      return 0;
    }
    else {
      check++;
    }
    check++;
  }

  return 1;
}

/*
 *  ͔֐
 *   true:OK / false:NG
 */
static bool_t is_ipaddr_str(uint8_t* s, uint8_t c)
{
  if ( ( c >= '0' &&  c <= '9' ) || c == ':' || c == '.' )
  {
    return true;
  }
  return false;
}

static bool_t is_enable(uint8_t* s, uint8_t c)
{
  int_t len;
  uint8_t **check = (uint8_t**)s_bool_name;
  uint8_t temp_buf[ORTMS_MAX_LEN + 4];

  len = s_strlen( s );
  s_strcpy( temp_buf, s );
  temp_buf[len] = c;
  len++;
  temp_buf[len] = '\0';

  while ( *check != NULL ) {

    if ( s_strncmp( *check, temp_buf, len ) == 0 ) {
      return true;
    }
    check++;
  }

  return false;
}

static bool_t is_log_level(uint8_t* s, uint8_t c)
{
  int_t len;
  uint8_t **check = (uint8_t**)s_level_name;
  uint8_t temp_buf[68];

  len = s_strlen( s );
  s_strcpy( temp_buf, s );
  temp_buf[len] = c;
  len++;
  temp_buf[len] = '\0';

  while ( *check != NULL ) {

    if ( s_strncmp( *check, temp_buf, len ) == 0 ) {
      return true;
    }
    check++;
  }

  return false;
}

static bool_t is_number(uint8_t* s, uint8_t c)
{
  if ( ( c >= '0' &&  c <= '9' ) || c == '.' )
  {
    return true;
  }
  return false;
}

static const struct ParamInfo s_param_info[] = {
  { (uint8_t *)"corba.endpoint"        , is_ipaddr_str },
  { (uint8_t *)"corba.nameservers"     , is_ipaddr_str },
  { (uint8_t *)"naming.formats"        , NULL },
  { (uint8_t *)"logger.enable"         , is_enable },
  { (uint8_t *)"logger.log_level"      , is_log_level },
  { (uint8_t *)"exec_cxt.periodic.rate", is_number },
  { (uint8_t *)"timer.tick"            , is_number },
  { (uint8_t *)"\0", NULL }
};

/*-----------------------------------------------------------
	Public Funcitons
-----------------------------------------------------------*/
/*!
 * @if jp
 * @brief OpenRTM̐ݒs
 * @else
 * @endif
 */
void OpenRTMSetup(){

  int32_t   current;
  int32_t   cnt;
  uint32_t  buff_addr = (uint32_t)&end;
  int_t i;
  uint8_t  *w_addr = (uint8_t *)(buff_addr + FLASH_MAGICWORD_SIZE);
  						//ݗpAhX
  uint8_t  *r_addr = (uint8_t *)FLASH_CONFIG_ADDR + FLASH_MAGICWORD_SIZE;
  						//ǂݍݗpAhX
  uint8_t   work;
  int8_t    isChange = 0;
  uint32_t  lop;
  uint32_t  delete_addr = (uint32_t)&end;
  uint8_t   default_value[ORTMS_MAX_LEN];
  uint8_t confbuf[ORTMS_MAX_LEN];
  int_t n;

  //obt@̈NA
  for ( lop = 0; lop < FLASH_CONFIG_SIZE; lop++ )
  {
    *(uint8_t *)delete_addr = 0;
    delete_addr += sizeof(uint8_t);
  }

  //VAʐM
#ifndef FLASH_DEBUG
  ep93xx_init_uart();
#endif

  // ݒσf[^̊mF
  if ( checkMagicWordForFlash() )
  {
    isChange = 0;
  }
  else
  {
    isChange = 1;
  }

  // }WbN[h̏
  setMagicWord();

  //RtBOϐJԂ
  for(current = 0; s_param_info[current].name[0] != '\0'; current++){
    //ϐo
    s_puts(s_param_info[current].name);
    cnt = s_strcpy((uint8_t*)w_addr, s_param_info[current].name);
    w_addr += cnt;
    r_addr += cnt;
    *(uint8_t *)w_addr = ':';
    w_addr += sizeof(uint8_t);
    r_addr += sizeof(uint8_t);

    ep93xx_putc('(');

    default_value[0] = '\0';
    i = 0;

    //ݒl\
    //RtBOɒlȂ͎sȂ
    if(isChange == 0){
      for (i = 0, work = r_addr[i]; work != '\n' && work != '\0'; work = r_addr[++i]) {
        if(work != '\n' && work != '\r' && work != '\0'){
          s_putc(work);
        }
      }
    }

    if ( i == 0 ) {
      s_copy_default( s_param_info[current].name, default_value );
      lop = 0;
      while( default_value[lop] != '\0' ) {
        s_putc(default_value[lop]);	
        lop++;
      }
    }

    ep93xx_putc(')');
    ep93xx_putc(':');
    ep93xx_putc(' ');

    /* read command */
    for (n=0; ; /* nada */) {
      work = ep93xx_getc();
      if (work == BS || work == DEL) {
        /* backspace and rubout */
	if (n > 0) {
	  --n;
	  s_putc(BS);
	  s_putc(' ');
	  s_putc(BS);
	}
      } else if (work == CR || work == LF) {
	/* end of command line */
	break;
      } else if (work < ' ') {
	/* control character */
	s_putc('^');
	s_putc(work + '@');
      } else if (n < (sizeof confbuf)-1) {
	/* ordinary character; echo. */
	/* ͒l̃`FbN */
	confbuf[n] = '\0';
	if (s_param_info[current].param_valid ==NULL ||
	    s_param_info[current].param_valid(confbuf, work)) {
	  s_putc(work);
	  confbuf[n++] = work;
	}
      }
    }
    confbuf[n] = '\0';

    if (confbuf[0] != '\0') {
      for (i = 0; confbuf[i] != '\0'; i++, w_addr++) {
        *w_addr = confbuf[i];
      }
      *(uint8_t *)w_addr = '\n';
      w_addr++;
      /* ݒl̓ǂݔ΂ */
      while (*r_addr != '\n') {
        r_addr++;
      }
      r_addr++;				/* sǂݔ΂ */
    } else {
      if(isChange == 1 || ( isChange != 1 && *r_addr == '\n' ) ) {
        lop = 0;
        while( default_value[lop] != '\0' ) {
          *w_addr = default_value[lop];
          w_addr++;
          lop++;
        }
        *w_addr = '\n';

        w_addr++;
        /* sǂݔ΂ */
        r_addr++;
      }
      else{
        for (i = 0; *r_addr != '\n'; i++, w_addr++, r_addr++) {
          *w_addr = *r_addr;
        }
        *w_addr = *r_addr;
        w_addr++;
        r_addr++;
      }
    }
    s_putc('\n');
  }

  *w_addr = EOF;
  //obt@RtBÖփRs[
  flash_writer(buff_addr, FLASH_CONFIG_ADDR,FLASH_BASE_ADDR,0x2000);

  s_puts((uint8_t *)"complete!\n");
}

/*!
 * @if jp
 * @brief tbVւ݂̏s
 * @else
 * @endif
 */
void
flash_writer(uint32_t from, uint32_t to, uint32_t base, uint32_t size){

  int32_t   cnt;
  intptr_t  timeout;  //ԊԊuǗ
  uint16_t  *ptr;   //݌AhXێ
  uint32_t  addr;   //ݐAhXێ

  ptr = (uint16_t *)from;
  cnt = 0;

#ifdef FLASH_DEBUG
  loc_cpu();
#endif

  //f[^SďĂ邩ׂ
  for( addr = to; *(volatile uint16_t *)( addr ) == 0xFFFF; addr += (sizeof(int16_t) * 2)){
    if(addr >= to + WRITE_SIZE){
      break;
    }
  }

  //f[^܂Ăꍇ͏
  if(addr < (uintptr_t)to + WRITE_SIZE){
    *(volatile uint16_t *)((to) + (0x555 << 1)) = 0xAA;
    *(volatile uint16_t *)((to) + (0x2AA << 1)) = 0x55;
    *(volatile uint16_t *)((to) + (0x555 << 1)) = 0x80;
    *(volatile uint16_t *)((to) + (0x555 << 1)) = 0xAA;
    *(volatile uint16_t *)((to) + (0x2AA << 1)) = 0x55;
    *(volatile uint16_t *)(to) = 0x30;
    
    //f̂҂
    timeout = 40000000;
    addr = to;
    while (timeout > 0) {
      for(;addr < to + WRITE_SIZE; addr += (sizeof(int16_t) * 2)){
        if((*(volatile uint16_t *)( addr ) != 0xFFFF)){
          break;
        }
      }
      if(addr >= to + WRITE_SIZE){
        break;
      }
      timeout--;
    }
  }

  //݂s
  for(addr = to; addr < to + WRITE_SIZE; addr += (sizeof(short) * 2)) 
  {
    *(volatile uint16_t *)(base + (0x555 << 1)) = 0xAA;
    *(volatile uint16_t *)(base + (0x2AA << 1)) = 0x55;
    *(volatile uint16_t *)(base + (0x555 << 1)) = 0xA0;
    *(volatile uint16_t *)(addr) = *ptr;

    timeout = 40000000;
    while ((((*(volatile uint16_t *)(addr))) != *ptr) && timeout > 0) {
      timeout--;
    }

    *(volatile uint16_t *)(base + (0x555 << 1)) = 0xAA;
    *(volatile uint16_t *)(base + (0x2AA << 1)) = 0x55;
    *(volatile uint16_t *)(base + (0x555 << 1)) = 0xA0;
    *(volatile uint16_t *)(addr + 2) = *(ptr + 1);

    timeout = 40000000;
    while ((((*(volatile uint16_t *)(addr + 2))) != *(ptr+1)) && timeout > 0) {
      timeout--;
    }
    ptr += 2;
  }

#ifdef FLASH_DEBUG
  unl_cpu();
#endif

}

/*!
 * @if jp
 * @brief ݒf[^̗L̊mF(0:A1:L)
 * @else
 * @endif
 */
uint32_t
checkMagicWordForFlash( void )
{
  uint32_t  r_addr = FLASH_CONFIG_ADDR; //ǂݍݗpAhXێ
  uint32_t  index;

  for( index = 0; index < FLASH_MAGICWORD_SIZE; index++ )
  {
    if (*(int8_t *)(r_addr) != s_magic_word[ index ] ) 
    {
      return 0;
    }
    r_addr += sizeof(uint8_t);
  }
  return (1);
}

/*!
 * @if jp
 * @brief }WbN[h̐ݒ
 * @else
 * @endif
 */
void
setMagicWord( void )
{
  uint32_t  w_addr = (uint32_t)&end;
  uint32_t  index;

  for( index = 0; index < FLASH_MAGICWORD_SIZE; index++ )
  {
    *(int8_t *)(w_addr) = s_magic_word[ index ];
    w_addr += sizeof(uint8_t);
  }
}

/*!
 * @if jp
 * @brief l[T[oAhX̎擾
 * @else
 * @endif
 */
void
getNameServerAddr( int8_t* addr )
{
  uint8_t  *r_addr = (uint8_t *)FLASH_CONFIG_ADDR + FLASH_MAGICWORD_SIZE;
  						//ǂݍݗpAhX
  int_t len = 0;

  if ( addr == NULL ) {
    return;
  }

  *addr = '\0';
  s_copy_default( s_name_server_param_name, (uint8_t *)addr );

  if ( checkMagicWordForFlash() == 0 ) {
    return;
  }

  len = s_strlen( (uint8_t *)s_name_server_param_name );

  while( *r_addr != EOF ) {
    if ( s_strncmp( (uint8_t *)s_name_server_param_name, r_addr, len ) == 0 )
    {
      r_addr += ( len + 1 );
      if ( *r_addr == '\n' || *r_addr == EOF ) {
        return;
      }

      while ( *r_addr != '\n' && *r_addr != EOF ) {
        *addr = *r_addr;
        addr++;
        r_addr++;
      }
      *addr = '\0';
      return;
    }
    while( *r_addr != EOF ) {
      if ( *r_addr == '\n' ) {
        r_addr++;
        break;
      }
      r_addr++;
    }
  }
}

/*!
 * @if jp
 * @brief RtBO[Ṽf[^e𕶎Ŏ擾
 * @else
 * @endif
 */
bool_t
getConfigContent( uint8_t const * pname, uint8_t* cdata, int_t clen )
{
  uint8_t* pos = (uint8_t *)GET_CONFIG_DATA_STR();
  int_t plen;
  bool_t detected = false;
  int_t i;

  /* p[^`FbN */
  if (!pname) {
    return false;
  }
  /* }WbNio[̃`FbN */
  if ( checkMagicWordForFlash() == 0 ) {
    return false;
  }

  /* f[^ẽftHglݒ */
  cdata[0] = '\0';
  s_ncopy_default( pname, cdata, clen );

  plen = s_strlen( (uint8_t *)pname );

  while (*pos != EOF && !detected) {
    if ( s_strncmp( (uint8_t *)pname, pos, plen ) == 0 &&
    	pos[plen] == DLM_CFG ) {
      /* Ώۂ̍ڌ */
      detected = true;
      pos += (plen + 1);		/* Rec̐擪Ɉړ */
      /* Recɕ */
      for ( i = 0; i < clen - 1; i++, pos++ ) {
        if ( *pos == LF || *pos == EOF ) {
          break;
        }
        cdata[i] = *pos;
      }
      cdata[i] = '\0';
    } else {
      /* ̍sցAǂݔ΂ */
      while ( *pos != LF ) {
        pos++;
      }
      pos++;
    }
  }

 return detected;
}
