/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2008 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2006-2008 by Task-intelligence Research Group,
 *             Intelligent Systems Research Institute,
 *             National Institute of
 *               Advanced Industrial Science and Technology (AIST), Japan
 * 
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include <itron.h>
#include <kernel.h>
#include <tinet_defs.h>
#include <tinet_config.h>
#include <net/if.h>
#include <net/ethernet.h>
#include <net/net_buf.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>
#include "ethmac.h"
#include "stringlib.h"

typedef long time_t;
typedef long suseconds_t;
struct timeval {
	time_t		tv_sec;		/* b */
	suseconds_t	tv_usec;	/* }CNb */
};

#define SEC(systim) (systim / 1000)
#define USEC(systim) ((systim % 1000) * 1000)

#define MAX_IP_ADDR_STR	15
#define MAX_DLM_CNT	3

uint8_t *get_ifaddr(void)
{
  T_IFNET *ifp = IF_GET_IFNET();
  return (uint8_t*)IF_SOFTC_TO_IFADDR(ifp->ic);
}

/*
 *  IPAhXƃTulbg}XN̐ݒE擾
 */
uint32_t get_ipaddr(void)
{
  T_IFNET *ifp = IF_GET_IFNET();

  return (uint32_t)ifp->in_ifaddr.addr;
}

void set_ipaddr(uint32_t addr)
{
  T_IFNET *ifp = IF_GET_IFNET();

  ifp->in_ifaddr.addr = (T_IN4_ADDR)addr;

  /*  [eBOe[ũ^[QbgAhX̍XV
   *    ڑM̐ݒeXV
   *    }XNl͒Pɒ`l̂܂܎gp
   */
  routing_tbl[1].target = ( addr & IPV4_ADDR_LOCAL_MASK );
}

uint32_t get_maskaddr(void)
{
  T_IFNET *ifp = IF_GET_IFNET();

  return (uint32_t)ifp->in_ifaddr.mask;
}

void set_maskaddr(uint32_t addr)
{
  T_IFNET *ifp = IF_GET_IFNET();

  ifp->in_ifaddr.mask = (T_IN4_ADDR)addr;
}

int32_t
_gettimeofday (struct timeval * tp, void * tzvp)
{
  SYSTIM systim;
  ER ercd = get_tim(&systim);
  if (ercd >= 0) {
    tp->tv_sec = SEC(systim);
    tp->tv_usec = USEC(systim);
  }
  return (int32_t)ercd;
}

void set_endp_portno(uint16_t port)
{
  tcp_rep[0].myaddr.portno = port;
}

uint16_t get_endp_portno()
{
  return tcp_rep[0].myaddr.portno;
}

/*
 *  (IP addr):(port)`̕񂩂IP addr,port擾
 *  IP addrXX.XX.XX.XX̌`
 *  XXport10i
 */
bool_t unpack_ip_port( uint8_t const * str, uint32_t* ip, uint16_t* port )
{
  uint8_t* cur;
  uint8_t ip_str[ MAX_IP_ADDR_STR + 1 ];
  uint8_t* ip_pos[4];
  int_t dlm_cnt;
  int_t i;
  int_t tmp;

  cur = (uint8_t *)str;

  /* IPAhX̎擾 */
  if ( ip != NULL ) {
    dlm_cnt = 0;
    i = 0;
    ip_pos[0] = &ip_str[0];
    /* tH[}bg`FbNȂIPAhX܂ł擾 */
    while ( *cur != ':'  && *cur != '\0' && i < MAX_IP_ADDR_STR ) {
      if ( *cur == '.' ) {
        if ( dlm_cnt < MAX_DLM_CNT ) {
          ip_str[i++] = '\0';
          ip_pos[++dlm_cnt] = &ip_str[i];
        }
      } else {
        ip_str[i++] = *cur;
      }
      cur++;
    }
    if ( *cur == '\0' || dlm_cnt != MAX_DLM_CNT ||
	( *cur != ':' && i >= MAX_IP_ADDR_STR ) ) {
      /* tH[}bgG[ */
      /* @؂蕶Ȃ@A؂蕶RłȂ@Bz */
      return false;
    }
    ip_str[i] = '\0';

    *ip = 0;
    for ( i = 0; i < MAX_DLM_CNT + 1; i++ ) {
      tmp = _atoi( (char*)ip_pos[i] );
      /* IPAhX͈̔͊mF */
      if ( tmp < 0 || tmp > 255 ) {
        return false;
      }
      *ip <<= 8;
      *ip |= (uint8_t)tmp;
    }
  }

  /* |[gԍ̎擾 */
  if ( port != NULL ) {
    *port = 0;
    /* IPAhXƃ|[gԍ̋؂蕶܂œǂݔ΂ */
    while ( *cur != ':' && *cur != '\0' ) {
      cur++;
    }
    if ( *cur == '\0' ) {
      /* tH[}bgG[ */
      /* ؂蕶Ȃ */
      return false;
    }
    cur++;
    /* I[NULLȂ̂ŁÂ܂ܐlϊ */
    tmp = _atoi( (char*)cur );
    /* |[gԍ͈̔͊mF */
    if ( tmp < 0 || tmp > 65535 ) {
      return false;
    }
    *port = (uint16_t)tmp;
  }

  return true;
}
