/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2008 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2006-2008 by Task-intelligence Research Group,
 *             Intelligent Systems Research Institute,
 *             National Institute of
 *               Advanced Industrial Science and Technology (AIST), Japan
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  @(#) $Id: $
 */

#include <kernel.h>
#include "stringlib.h"

#define FC_MINUS (1)
#define FC_PLUS  (2)
#define FC_ZERO  (4)
#define FC_BLANK (8)
#define FC_SHARP (16)

static char* rvers_str(char* buf, int len)
{
	char tmp;
	int i;

	for (i = 0; i < len/2 ;i++) {
		tmp = buf[i];
		buf[i] = buf[len - 1 - i];
		buf[len - 1 - i] = tmp;
	}

	return buf;
}

static const char num_cvt[] = "0123456789ABCDEF";
char* itoa(long val, char *buf, int radix)
{
	int i;
	unsigned long quot, rest;

	if (radix > 16) {
		return NULL;
	}

	if (val < 0 && radix == 10) {
		quot = 0 - val;
	} else {
		quot = val;
	}
	rest = quot % radix;
	buf[0] = num_cvt[rest];
	quot /= radix;
	for (i = 1; quot > 0; i++) {
		rest = quot % radix;
		buf[i] = num_cvt[rest];
		quot /= radix;
	}
	if (val < 0 && radix == 10) {
		buf[i++] = '-';
	}
	buf[i] = '\0';

	return rvers_str(buf, i);
}

char* uitoa(unsigned long val, char* buf, int radix)
{
	int i;
	unsigned long quot, rest;

	if (radix > 16) {
		return NULL;
	}
	quot = val;
	rest = quot % radix;
	buf[0] = num_cvt[rest];
	quot /= radix;
	for (i = 1; quot > 0; i++) {
		rest = quot % radix;
		buf[i] = num_cvt[rest];
		quot /= radix;
	}
	buf[i] = '\0';

	return rvers_str(buf, i);
}

static int _isdigit(const char c)
{
  return ((c>='0')&&(c<='9'));
}

static int _atoi_ptr(const char **ptr)
{
  long l;
  int minus;
  const char *p;
  p = *ptr;

  l = 0L;
  minus = 0;
  if(*p == '-') {
	minus = 1;
	p++;
  }

  while(_isdigit(*p)){
	l *= 10;
	l += (*p-'0');
	p++;
  }
  if(minus) l = -l;
  *ptr = p; /* ᤷ */
  /* printf("debug: %ld, %s\n", l, p); */
  return (int)l;
}

int _atoi(const char* ptr) {
  return _atoi_ptr((const char **)&ptr);
}

static int ntorstru(char *buf, unsigned long num, int base)
{
  unsigned long l, m;
  char *b0;
  char *cvt;
  char *cvt0 = "0123456789abcdef";
  char *cvt1 = "0123456789ABCDEF";

  b0 = buf;
  cvt = cvt0;

  if(base<0) {
    cvt= cvt1;
    base = -base;
  }

  l = num;
  m = 0;

  if(l==0) *buf++ = '0';
  while(l) {
    m = l % base;
    l /= base;
    *buf++ = cvt[(int)m];
  }
  *buf = '\0';
  return buf-b0;
}

int t_strlen(char *s)
{
  int l;
  l = 0;
  while(*s++) l++;
  return l;
}

static int _t_vsnprintf(char *buffer, unsigned long count,
		const char *format, int *argptr)
/*
  buffer
    ϤγǼ
  count
    񤭹ʸ
  format
    񼰤λ
  argptr
    ꥹȤؤΥݥ
*/
{
  char *ptr, *eob;
  unsigned flags;       /* format control : flag characters */
  int ef;
  const char *pf;
  int width;
  int precision;
  int longf;
  int len, len2;
  char *s;
  int i;
  char numbuf[30];
  int symbol, minus;
  long v;
  unsigned long uv;
  int c = 0;

  ptr = buffer;         /* current output pointer */
  eob = (char *)(ptr + count -1); /* end of buffer */

  pf = format;
  while(*pf){
    if(ptr >= eob) break;

    if(*pf != '%') {
      *ptr++ = *pf++;
      continue;
    }

    pf++; /* skip '%' */

    /* correct flags */
    flags = 0;
    ef = 0;
    while(!ef){
      switch(*pf){
        case '-':
          flags |= FC_MINUS;
          break;
        case '+':
          flags |= FC_PLUS;
          break;
        case '0':
          flags |= FC_ZERO;
          break;
        case ' ':
          flags |= FC_BLANK;
          break;
        default:
          ef = 1;
          break;
      }
      if(!ef) pf++;
    }

    /* width */
    width = -1; /* none */
    if(_isdigit(*pf)) width = _atoi_ptr(&pf);

    precision = -1; /* none */
    if(*pf == '.') {
      pf++;
      precision = _atoi_ptr(&pf);
    }

    /* size flag */
    longf = 0;
    if(*pf == 'l') {
      pf++;
      longf = 1;
    }

    /* output */
    switch(*pf++) {
      case '%':
        if(ptr < eob) *ptr++ = '%';
        continue;
        /* break; */

      case 'c':
        if(ptr < eob) *ptr++ = (char)argptr[c];
        c++;
        continue;
        /* break; */

      case 's':
        /* printf("debug: s\n"); */
        s = (char*)argptr[c];
        c++;
        len = t_strlen(s);
        /* width.precision ˤĴ */
        if((precision != -1)&&(len > precision)) len = precision;
        if(width == -1) width = len;
        if(width < len) width = len; /* Ϥ */

        if(!(flags & FC_MINUS)) { /* ͤξ */
          for(i = 0; i < width - len; i++) {
            if(ptr < eob) *ptr++ = ' ';
          }
        }
        for(i = 0; i < len; i++) *ptr++ = *s++; /* ʸʬ */
        if((flags & FC_MINUS)) { /* ͤξ */
          for(i = 0; i < width - len; i++) {
            if(ptr < eob) *ptr++ = ' ';
          }
        }
        continue;
        /*break;*/


      case 'd':
      case 'i':
        if(longf) {
          v = (long)argptr[c];
          c++;
        } else {
          v = (int)argptr[c];
          c++;
        }
        minus = (v < 0)?1:0;
        if(minus) v = -v;
        len = ntorstru(numbuf, v, 10);
        break;

      case 'u':
        if(longf) {
          uv = (unsigned long)argptr[c];
          c++;
        } else {
          uv = (unsigned int)argptr[c];
          c++;
        }
        len = ntorstru(numbuf, uv, 10);
        minus = 0;
        break;

      case 'o':
        if(longf) {
          uv = (unsigned long)argptr[c];
          c++;
        } else {
          uv = (unsigned int)argptr[c];
          c++;
        }
        len = ntorstru(numbuf, uv, 8);
        minus = 0;
        break;

      case 'x':
        if(longf) {
          uv = (unsigned long)argptr[c];
          c++;
        } else {
          uv = (unsigned int)argptr[c];
          c++;
        }
        len = ntorstru(numbuf, uv, 16);
        minus = 0;
        break;

      case 'X':
        if(longf) {
          uv = (unsigned long)argptr[c];
          c++;
        } else {
          uv = (unsigned int)argptr[c];
          c++;
        }
        len = ntorstru(numbuf, uv, -16); /* base<0a-fʸ*/
        minus = 0;
        break;

      default:
        /* ե饰Ȳᤵ줿Τʤ */
        if(ptr < eob) *ptr++ = '%';
        if(ptr < eob) *ptr++ = *(pf-1);
        continue;
        /*break;*/
    }

    /* ʲϿͤΤ */

    len2 = len; /* ʬޤ᤿ϤɬפĹ */
    symbol = 0; /* ʬ */
    if(minus) {
      symbol = '-';
    } else {
      if(flags & FC_BLANK) symbol = ' ';
      if(flags & FC_PLUS)  symbol = '+'; /* ' '񤭤 */
    }
    if(symbol) len2++;

    if(width < precision) width = precision; /* width.precision ̤ƤʤĤǤΤ */
    if(width < len2) width = len2;           /* ɬפ˳ĥ(ʤޤ) */

    if(!(flags & FC_MINUS)) { /* 󤻤Ǥʤ */
      if(flags & FC_ZERO) { /* ZERO-FILL */
        if(symbol) {
          if(ptr < eob) *ptr++ = symbol;              /* ĤƤ */
        }
        for(i=0;i<width-len2;i++) {
          if(ptr < eob) *ptr++ = '0';  /* ֤0 */
        }
      } else {                 /* 󤻤ξ */
        for(i=0;i<width-len2;i++) {
          if(ptr < eob) *ptr++ = ' ';  /* ֤Ƥ */
        }
        if(symbol) {
          if(ptr < eob) *ptr++ = symbol;              /* Ĥ */
        }
      }
    } else {
      if(symbol) {
        if(ptr < eob) *ptr++ = symbol;                /* ɬפʤ */
      }
    }
    for(i = len - 1; i >= 0; i--) {
      if(ptr < eob) *ptr++ = numbuf[i]; /* (սʸ)Ϥ */
    }
    if(flags & FC_MINUS) { /* 󤻤꤬жϤ */
      for(i = 0; i < width - len2; i++) {
        if(ptr < eob) *ptr++ = ' ';
      }
    }

    /* continue; */
  } /* while(*pf) */
  *ptr = '\0';
  return (ptr - buffer); /* ϤʸĹ */
}

int t_vsnprintf_1(char *buffer, unsigned long count,
		const char *format, int a1)
{
   int args[1] = {a1};
   return _t_vsnprintf(buffer, count, format, args);
}

int t_vsnprintf_2(char *buffer, unsigned long count,
		const char *format, int a1, int a2)
{
   int args[2] = {a1, a2};
   return _t_vsnprintf(buffer, count, format, args);
}

int t_vsnprintf_3(char *buffer, unsigned long count,
		const char *format, int a1, int a2, int a3)
{
   int args[3] = {a1, a2, a3};
   return _t_vsnprintf(buffer, count, format, args);
}

int t_vsnprintf_4(char *buffer, unsigned long count,
		const char *format, int a1, int a2, int a3, int a4)
{
   int args[5] = {a1, a2, a3, a4};
   return _t_vsnprintf(buffer, count, format, args);
}

int t_vsnprintf_5(char *buffer, unsigned long count,
		const char *format, int a1, int a2, int a3, int a4, int a5)
{
   int args[5] = {a1, a2, a3, a4, a5};
   return _t_vsnprintf(buffer, count, format, args);
}

int t_vsnprintf_6(char *buffer, unsigned long count,
		const char *format, int a1, int a2, int a3, int a4, int a5, int a6)
{
   int args[6] = {a1, a2, a3, a4, a5, a6};
   return _t_vsnprintf(buffer, count, format, args);
}

#define STDOUT_PORTID		1		/* ɸѤΥꥢݡֹ */
static void stdout_putc(char_t c)
{
  serial_wri_dat(STDOUT_PORTID, &c, 1);
}

void t_printf_0(const char *format)
{
  syslog_printf(format, NULL, stdout_putc);
}

void t_printf_1(const char *format, int a1)
{
  int args[1] = {a1};
  syslog_printf(format, args, stdout_putc);
}

void t_printf_2(const char *format, int a1, int a2)
{
  int args[2] = {a1, a2};
  syslog_printf(format, args, stdout_putc);
}

void t_printf_3(const char *format,
	int a1, int a2, int a3)
{
  int args[3] = {a1, a2, a3};
  syslog_printf(format, args, stdout_putc);
}

void t_printf_4(const char *format,
	int a1, int a2, int a3, int a4)
{
  int args[5] = {a1, a2, a3, a4};
  syslog_printf(format, args, stdout_putc);
}

void t_printf_5(const char *format,
	int a1, int a2, int a3, int a4, int a5)
{
  int args[5] = {a1, a2, a3, a4, a5};
  syslog_printf(format, args, stdout_putc);
}

void t_printf_6(const char *format,
	int a1, int a2, int a3, int a4, int a5, int a6)
{
  int args[6] = {a1, a2, a3, a4, a5, a6};
  syslog_printf(format, args, stdout_putc);
}
