/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2008 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2006-2008 by Task-intelligence Research Group,
 *             Intelligent Systems Research Institute,
 *             National Institute of
 *               Advanced Industrial Science and Technology (AIST), Japan
 * 
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include <string.h>
#include <kernel.h>
#include <sil.h>
#include "ep93xx.h"
#include "switch.h"

#define CONFIG_SW_NUM	(2)
/*-----------------------------------------------------------
	Definitions
-----------------------------------------------------------*/

/* Switch Configuration. */
#define BIT(x)			(1<<(x))
#define SW_POLL_NUM		4

#ifdef CONFIG_SW_NUM
#define USER_SW_NUM		CONFIG_SW_NUM
#else
#define USER_SW_NUM		8
#endif /* CONFIG_SW_NUM */

/*
 * SW GPIO Port Number
 * SW1 GPIO_A:4
 * SW2 GPIO_A:5
 */
#define BTN_SW1			(0x01 << 4)
#define BTN_SW2			(0x01 << 5)

/*-----------------------------------------------------------
	Private Variables
-----------------------------------------------------------*/
static char_t first_state = 0;
static char_t SwPortStatus[USER_SW_NUM][SW_POLL_NUM];
static char_t SwPortPos;
static char_t PreSwState = 0;
static char_t CurSwState;
static const char_t SwPort[] = {
	BTN_SW1, BTN_SW2
};

typedef void (*SW_FUNC)(void);
typedef struct _SW_FUNC_TBL {
	SW_FUNC on;
	SW_FUNC off;
} SW_FUNC_TBL;

SW_FUNC_TBL const sw_func_table[] = {
	{ sw1_on, sw1_off },
#if USER_SW_NUM >= 2
	{ sw2_on, sw2_off },
#endif
};

/*-----------------------------------------------------------
	Private Funcitons
-----------------------------------------------------------*/

/*-----------------------------------------------------------
	Public Funcitons
-----------------------------------------------------------*/
void init_switch(void)
{
	int_t i;

	first_state = 0;

	/* SW|[g͐ݒ */
	sil_andw((void*)GPIO_PADDR, ~(BTN_SW1 | BTN_SW2));

	CurSwState = PreSwState = 0;
	memset(SwPortStatus, -10, sizeof(SwPortStatus));
	SwPortPos = 0;
}

void watch_switch(void)
{
	char_t SwStatus;
	char_t SwEvent;
	int_t i, j;

	for (i = 0; i < USER_SW_NUM; i++) {
		SwPortStatus[i][SwPortPos & 0x03] =
			((sil_rew_mem((void*)GPIO_PADR) & SwPort[i]) ? 0 : 1);
	}
	SwPortPos++;

	for (i = 0; i < USER_SW_NUM; i++) {
		for (j = 0, SwStatus = 0; j < SW_POLL_NUM; j++) {
			SwStatus += SwPortStatus[i][j];
		}
		if (SwStatus == SW_POLL_NUM) {
			/* SW OFF */
			CurSwState |= BIT(i);
		} else if (SwStatus == 0) {
			/* SW ON */
			CurSwState &= ~BIT(i);
		}
	}

	if (first_state == 0) {
		first_state++;
		SwEvent = CurSwState;
	}else {
		SwEvent = (CurSwState ^ PreSwState);
		if (SwEvent == 0) {
			/* SWԕωȂ */
			return;
		}
	}

	for (i = 0; i < USER_SW_NUM; i++) {
		if (SwEvent & BIT(i)) {
			if (CurSwState & BIT(i)) {
				/* SW ON */
				sw_func_table[i].on();
			} else {
				/* SW OFF */
				sw_func_table[i].off();
			}
		}
	}

	PreSwState = CurSwState;
}

