/*
 *  TINET (TCP/IP Protocol Stack)
 * 
 *  Copyright (C) 2006-2008 by Task-intelligence Research Group,
 *             Intelligent Systems Research Institute,
 *             National Institute of
 *               Advanced Industrial Science and Technology (AIST), Japan
 *
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́C̏𖞂ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include <kernel.h>
#include <sil.h>
#include "ep93xx.h"

#include "i2c_eeprom.h"

/*------------------------------------------------------------------------
	Definitions
------------------------------------------------------------------------*/
#define I2C_SDA_BIT		(0x01 << 4)
#define I2C_SCL_BIT		(0x01 << 5)

/*------------------------------------------------------------------------
	Private Funcitons
------------------------------------------------------------------------*/
#define i2c_delay() sil_dly_nse(10000)

static void i2c_set_sda(void)
{
	sil_orw((void*)GPIO_PBDDR, I2C_SDA_BIT);
	sil_orw((void*)GPIO_PBDR, I2C_SDA_BIT);

	i2c_delay();
}

static void i2c_clr_sda(void)
{
	sil_orw((void*)GPIO_PBDDR, I2C_SDA_BIT);
	sil_andw((void*)GPIO_PBDR, ~I2C_SDA_BIT);

	i2c_delay();
}

static int_t i2c_read_sda(void)
{
	uint32_t val;

	sil_andw((void*)GPIO_PBDDR, ~I2C_SDA_BIT);
	val = sil_rew_mem((void*)GPIO_PBDR) & I2C_SDA_BIT;

	i2c_delay();

	return (val ? 1 : 0);
}

static void i2c_set_scl(void)
{
	sil_orw((void*)GPIO_PBDDR, I2C_SCL_BIT);
	sil_orw((void*)GPIO_PBDR, I2C_SCL_BIT);

	i2c_delay();
}

static void i2c_clr_scl(void)
{
	sil_orw((void*)GPIO_PBDDR, I2C_SCL_BIT);
	sil_andw((void*)GPIO_PBDR, ~I2C_SCL_BIT);

	i2c_delay();
}

static int_t i2c_ack(void)
{
	int_t timeout = 1000;

	sil_andw((void*)GPIO_PBDDR, ~I2C_SDA_BIT);
	i2c_set_scl();

	while(timeout--) {
		if (i2c_read_sda() == 0) {
			break;
		}
	}
	if (timeout < 0) {
		return -1;
	}

	i2c_clr_scl();

	return 0;
}

static int_t i2c_read(void)
{
	uint8_t ch;
	uint8_t bit;  
	int_t i;

	for (i=0, ch=0; i < 8; i++) {
		i2c_set_scl();
		bit = i2c_read_sda();
		i2c_clr_scl();

		ch |= (bit << (7-i));
	}

	return (int_t)ch;
}

static void i2c_write(uint8_t ch)
{
	uint8_t bit;
	int_t i;

	i2c_clr_scl();

	for (i=0; i < 8; i++) {
		bit = ((ch >> (7-i)) & 0x01);
		if (bit) {
			i2c_set_sda();
		} else {
			i2c_clr_sda();
		}
		i2c_set_scl();
		i2c_clr_scl();
	}
}

static void i2c_start(void)
{
	i2c_set_sda();
	i2c_set_scl();
	i2c_clr_sda();
	i2c_clr_scl();
}

static void i2c_stop(void)
{
	i2c_clr_scl();
	i2c_clr_sda();
	i2c_set_scl();
	i2c_set_sda();
}

/*------------------------------------------------------------------------
	Public Functions
------------------------------------------------------------------------*/
void i2c_init(void)
{
	i2c_stop();
}

int_t eeprom_getc(uint32_t addr)
{
	int_t ch;

	i2c_start();

	//Control
	i2c_write(0xA0);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	//Address
	i2c_write((uint8_t)addr);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	i2c_start();

	//Control
	i2c_write(0xA1);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	//Data read
	ch = i2c_read();

	i2c_stop();

	return ch;
}

int eeprom_putc(uint32_t addr, uint_t ch)
{
	i2c_start();

	//Control
	i2c_write(0xA0);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	//Address
	i2c_write((uint8_t)addr);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	//Value
	i2c_write((uint8_t)ch);
	if(i2c_ack() == -1){
		i2c_stop();
		return -1;
	}

	i2c_stop();

	dly_tsk(10);

	return 0;
}

int_t get_mac_addr(uint8_t* addr)
{
	int_t i;
	int_t ch;

	i2c_init();

	for (i = 0; i < 6; i++) {
		ch = eeprom_getc(i);
		if (ch == -1) {
			return -1;
		}
		addr[i] = ch;
	}

	return 0;
}
